import Cocoa
import objc

gNumDaysInMonth = (0, 31, 28, 31, 30, 21, 30, 31, 31, 30, 31, 30, 31)


def isLeap(year):
    return ((year % 4) == 0 and ((year % 100) != 0)) or (year % 400) == 0


class CalendarMatrix(Cocoa.NSMatrix):
    lastMonthButton = objc.IBOutlet()
    monthName = objc.IBOutlet()
    nextMonthButton = objc.IBOutlet()

    __slots__ = ("_selectedDay", "_startOffset")

    def initWithFrame_(self, frameRect):
        self._selectedDay = None
        self._startOffset = 0

        cell = Cocoa.NSButtonCell.alloc().initTextCell_("")
        now = Cocoa.NSCalendarDate.date()

        cell.setShowsStateBy_(Cocoa.NSOnOffButton)
        self.initWithFrame_mode_prototype_numberOfRows_numberOfColumns_(
            frameRect, Cocoa.NSRadioModeMatrix, cell, 5, 7
        )

        count = 0
        for i in range(6):
            for j in range(7):
                val = self.cellAtRow_column_(i, j)
                if val:
                    val.setTag_(count)
                count += 1

        self._selectedDay = Cocoa.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(  # noqa: B950
            now.yearOfCommonEra(),
            now.monthOfYear(),
            now.dayOfMonth(),
            0,
            0,
            0,
            Cocoa.NSTimeZone.localTimeZone(),
        )
        return self

    @objc.IBAction
    def choseDay_(self, sender):
        prevSelDate = self.selectedDay()
        selDay = self.selectedCell().tag() - self._startOffset + 1

        selDate = (
            Cocoa.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(
                prevSelDate.yearOfCommonEra(),
                prevSelDate.monthOfYear(),
                selDay,
                0,
                0,
                0,
                Cocoa.NSTimeZone.localTimeZone(),
            )
        )
        self.setSelectedDay_(selDate)
        self.highlightTodayIfVisible()

        if self.delegate().respondsToSelector_("calendarMatrix:didChangeToDate:"):
            self.delegate().calendarMatrix_didChangeToDate_(self, selDate)

    @objc.IBAction
    def monthChanged_(self, sender):
        thisDate = self.selectedDay()
        currentYear = thisDate.yearOfCommonEra()
        currentMonth = thisDate.monthOfYear()

        if sender is self.nextMonthButton:
            if currentMonth == 12:
                currentMonth = 1
                currentYear += 1
            else:
                currentMonth += 1
        else:
            if currentMonth == 1:
                currentMonth = 12
                currentYear -= 1
            else:
                currentMonth -= 1

        self.setSelectedDay_(
            Cocoa.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(
                currentYear, currentMonth, 1, 0, 0, 0, Cocoa.NSTimeZone.localTimeZone()
            )
        )
        self.refreshCalendar()
        self.choseDay_(self)

    def setSelectedDay_(self, newDay):
        self._selectedDay = newDay

    def selectedDay(self):
        return self._selectedDay

    def refreshCalendar(self):

        selDate = self.selectedDay()
        currentMonth = selDate.monthOfYear()
        currentYear = selDate.yearOfCommonEra()

        firstOfMonth = (
            Cocoa.NSCalendarDate.dateWithYear_month_day_hour_minute_second_timeZone_(
                currentYear, currentMonth, 1, 0, 0, 0, Cocoa.NSTimeZone.localTimeZone()
            )
        )
        self.monthName.setStringValue_(
            firstOfMonth.descriptionWithCalendarFormat_("%B %Y")
        )
        daysInMonth = gNumDaysInMonth[currentMonth]

        if (currentMonth == 2) and isLeap(currentYear):
            daysInMonth += 1

        self._startOffset = firstOfMonth.dayOfWeek()

        dayLabel = 1

        for i in range(42):
            cell = self.cellWithTag_(i)
            if cell is None:
                continue

            if i < self._startOffset or i >= (daysInMonth + self._startOffset):
                # blank out unused cells in the matrix
                cell.setBordered_(False)
                cell.setEnabled_(False)
                cell.setTitle_("")
                cell.setCellAttribute_to_(Cocoa.NSCellHighlighted, False)
            else:
                # Fill in valid days in the matrix
                cell.setBordered_(True)
                cell.setEnabled_(True)
                cell.setFont_(Cocoa.NSFont.systemFontOfSize_(12))
                cell.setTitle_(str(dayLabel))
                dayLabel += 1
                cell.setCellAttribute_to_(Cocoa.NSCellHighlighted, False)

        self.selectCellWithTag_(selDate.dayOfMonth() + self._startOffset - 1)
        self.highlightTodayIfVisible()

    def highlightTodayIfVisible(self):
        now = Cocoa.NSCalendarDate.date()
        selDate = self.selectedDay()

        if (
            selDate.yearOfCommonEra() == now.yearOfCommonEra()
            and selDate.monthOfYear() == now.monthOfYear()
            and selDate.dayOfMonth() == now.dayOfMonth()
        ):
            aCell = self.cellWithTag_(now.dayOfMonth() + self._startOffset - 1)
            aCell.setHighlightsBy_(Cocoa.NSMomentaryChangeButton)
            aCell.setCellAttribute_to_(Cocoa.NSCellHighlighted, True)

    def awakeFromNib(self):
        self.setTarget_(self)
        self.setAction_("choseDay:")
        self.setAutosizesCells_(True)
        self.refreshCalendar()
        self.choseDay_(self)



import types

import objc
import Foundation
from PyObjCTest.testhelper import PyObjC_TestClass3
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSDictionarySubclassing(TestCase):
    # These tests seem to be specific for macOS
    def testExceptionInInit(self):
        if objc.platform != "MACOSX":
            return

        class DictTestExceptionClass(Foundation.NSDictionary):
            pass

        # Don't use self.assertRaises here, we once had a bug that
        # causes this to fail, while the assertRaises version would
        # (probably) have worked.
        import warnings

        warnings.filterwarnings("ignore", category=objc.UninitializedDeallocWarning)

        try:
            try:
                _ = DictTestExceptionClass.alloc().initWithDictionary_({})
                self.fail()
            except ValueError:
                pass
        finally:
            del warnings.filters[0]

    def testAnotherExceptionInInit(self):
        if objc.platform != "MACOSX":
            return

        class DictTestExceptionClass2(Foundation.NSDictionary):
            def initWithObjects_forKeys_count_(self, o, k, c):
                return objc.super(
                    DictTestExceptionClass2, self
                ).initWithObjects_forKeys_count_(o, k, c)

        import warnings

        warnings.filterwarnings("ignore", category=objc.UninitializedDeallocWarning)

        try:
            try:
                _ = DictTestExceptionClass2.alloc().initWithDictionary_({})
                self.fail()
            except ValueError:
                pass
        finally:
            del warnings.filters[0]

    def testExceptionInInitClsMeth(self):
        if objc.platform != "MACOSX":
            return

        class DictTestExceptionClass3(Foundation.NSDictionary):
            def initWithObjects_forKeys_count_(self, o, k, c):
                return objc.super(
                    DictTestExceptionClass3, self
                ).initWithObjects_forKeys_count_(o, k, c)

        try:
            _ = DictTestExceptionClass3.dictionaryWithDictionary_({})
            self.fail()
        except ValueError:
            pass


class TestNSDictionaryInteraction(TestCase):
    def testMethods(self):
        for nm in dir(dict):
            if nm.startswith("__"):
                continue

            if isinstance(
                getattr(dict, nm), (types.BuiltinFunctionType, types.FunctionType)
            ):
                # Skip class methods, that needs more work in the core
                continue

            self.assertTrue(
                hasattr(Foundation.NSMutableDictionary, nm),
                f"NSMutableDictionary has no method '{nm}'",
            )

    def testRepeatedAllocInit(self):
        for _ in range(1, 1000):
            _ = Foundation.NSDictionary.alloc().init()

    def testBasicInteraction(self):
        d = Foundation.NSMutableDictionary.dictionary()
        d["a"] = "foo"
        d["b"] = "bar"

        self.assertEqual(
            d["a"],
            "foo",
            "Failed to retrieve the same thing that was put into the dict.",
        )
        try:
            d["c"]
            self.fail("Should have raised...")
        except KeyError:
            pass

    def testPythonIteraction(self):
        d = Foundation.NSMutableDictionary.dictionary()
        d["a"] = "foo"
        d["b"] = "bar"

        k = list(d.keys())
        k.sort()
        self.assertTrue(k == ["a", "b"])

        k = list(d.values())
        k.sort()
        self.assertTrue(k == ["bar", "foo"])

        k = list(d.items())
        k.sort()
        self.assertTrue(k == [("a", "foo"), ("b", "bar")])

    def testIn(self):
        d = Foundation.NSMutableDictionary.dictionary()
        d["a"] = "foo"
        d["b"] = "bar"
        d[1] = "baz"
        d[0] = "bob"

        self.assertTrue("a" in d)
        self.assertTrue(1 in d)
        # self.assertTrue( -1 in d )
        # self.assertTrue( d[-1] is None )
        self.assertTrue("q" not in d)

        for k in d.allKeys():
            self.assertEqual(d.objectForKey_(k), d[k])

        for k in d:
            self.assertEqual(d.objectForKey_(k), d[k])

        del d["a"]
        self.assertTrue("a" not in d)

    def test_varargConstruction(self):
        u = Foundation.NSDictionary.dictionaryWithObjects_forKeys_(
            [1, 2, 3, 4], ["one", "two", "three", "four"]
        )
        v = Foundation.NSDictionary.alloc().initWithObjects_forKeys_(
            [1, 2, 3, 4], ["one", "two", "three", "four"]
        )
        w = Foundation.NSDictionary.dictionaryWithObjects_forKeys_count_(
            [1, 2, 3, 4, 5], ["one", "two", "three", "four", "five"], 4
        )
        x = Foundation.NSDictionary.alloc().initWithObjects_forKeys_count_(
            [1, 2, 3, 4, 5], ["one", "two", "three", "four", "five"], 4
        )
        y = Foundation.NSDictionary.dictionaryWithObjectsAndKeys_(
            1, "one", 2, "two", 3, "three", 4, "four", None
        )
        z = Foundation.NSDictionary.alloc().initWithObjectsAndKeys_(
            1, "one", 2, "two", 3, "three", 4, "four", None
        )

        self.assertEqual(len(u), 4)
        self.assertEqual(len(v), 4)
        self.assertEqual(len(w), 4)
        self.assertEqual(len(x), 4)
        self.assertEqual(len(y), 4)
        self.assertEqual(len(z), 4)

        self.assertEqual(u["one"], 1)
        self.assertEqual(v["two"], 2)
        self.assertEqual(w["three"], 3)
        self.assertEqual(x["one"], 1)
        self.assertEqual(y["two"], 2)
        self.assertEqual(z["four"], 4)

    def test_varargConstruction2(self):
        u = Foundation.NSMutableDictionary.dictionaryWithObjects_forKeys_(
            [1, 2, 3, 4], ["one", "two", "three", "four"]
        )
        self.assertIsNot(u, None)
        v = Foundation.NSMutableDictionary.alloc().initWithObjects_forKeys_(
            [1, 2, 3, 4], ["one", "two", "three", "four"]
        )
        self.assertIsNot(v, None)
        w = Foundation.NSMutableDictionary.dictionaryWithObjects_forKeys_count_(
            [1, 2, 3, 4, 5], ["one", "two", "three", "four", "five"], 4
        )
        self.assertIsNot(w, None)
        x = Foundation.NSMutableDictionary.alloc().initWithObjects_forKeys_count_(
            [1, 2, 3, 4, 5], ["one", "two", "three", "four", "five"], 4
        )
        self.assertIsNot(x, None)

        y = Foundation.NSMutableDictionary.dictionaryWithObjectsAndKeys_(
            1, "one", 2, "two", 3, "three", 4, "four", None
        )
        self.assertIsNot(y, None)
        z = Foundation.NSMutableDictionary.alloc().initWithObjectsAndKeys_(
            1, "one", 2, "two", 3, "three", 4, "four", None
        )
        self.assertIsNot(z, None)

        self.assertEqual(len(u), 4)
        self.assertEqual(len(v), 4)
        self.assertEqual(len(w), 4)
        self.assertEqual(len(x), 4)
        self.assertEqual(len(y), 4)
        self.assertEqual(len(z), 4)

        self.assertEqual(u["one"], 1)
        self.assertEqual(v["two"], 2)
        self.assertEqual(w["three"], 3)
        self.assertEqual(x["one"], 1)
        self.assertEqual(y["two"], 2)
        self.assertEqual(z["four"], 4)


class MyDictionaryBase(Foundation.NSDictionary):
    def count(self):
        if hasattr(self, "_count"):
            return self._count
        return -1

    def keyEnumerator(self):
        return None

    def objectForKey_(self, key):
        return None


class MyDictionary1(MyDictionaryBase):
    def initWithObjects_forKeys_count_(self, objects, keys, count):
        self._count = count
        self._objects = objects
        self._keys = keys
        return self


class MyDictionary2(MyDictionaryBase):
    def dictionaryWithObjects_forKeys_count_(self, objects, keys, count):
        if self is not MyDictionary2:
            raise AssertionError(self)
        return (objects, keys, count)


class TestSubclassing(TestCase):
    def testInitWithObjects(self):
        o = PyObjC_TestClass3.makeDictFromClass_method_(MyDictionary1, 1)

        self.assertIsInstance(o, MyDictionary1)
        self.assertEqual(o._count, 4)
        self.assertEqual(len(o._keys), 4)
        self.assertEqual(len(o._objects), 4)

    def testDictWithObjects(self):
        o = PyObjC_TestClass3.makeDictFromClass_method_(MyDictionary2, 0)

        self.assertIsInstance(o, tuple)
        self.assertEqual(o[2], 4)
        self.assertEqual(len(o[1]), 4)
        self.assertEqual(len(o[0]), 4)


class TestVariadic(TestCase):
    def testDictionaryWithObjectsAndKeys(self):
        o = Foundation.NSDictionary.dictionaryWithObjectsAndKeys_(42, "a", 43, "b")
        self.assertEqual(o, {"a": 42, "b": 43})
        self.assertIsInstance(o, Foundation.NSDictionary)

        o = Foundation.NSMutableDictionary.dictionaryWithObjectsAndKeys_(
            42, "a", 43, "b"
        )
        self.assertEqual(o, {"a": 42, "b": 43})
        self.assertIsInstance(o, Foundation.NSMutableDictionary)

    def testInitWithObjectsAndKeys(self):
        o = Foundation.NSDictionary.alloc().initWithObjectsAndKeys_(42, "a", 43, "b")
        self.assertEqual(o, {"a": 42, "b": 43})
        self.assertIsInstance(o, Foundation.NSDictionary)

        o = Foundation.NSMutableDictionary.alloc().initWithObjectsAndKeys_(
            42, "a", 43, "b"
        )
        self.assertEqual(o, {"a": 42, "b": 43})
        self.assertIsInstance(o, Foundation.NSMutableDictionary)


class TestNSDictionary(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSDictionary.isEqualToDictionary_)
        self.assertResultIsBOOL(Foundation.NSDictionary.writeToFile_atomically_)
        self.assertArgIsBOOL(Foundation.NSDictionary.writeToFile_atomically_, 1)
        self.assertResultIsBOOL(Foundation.NSDictionary.writeToURL_atomically_)
        self.assertArgIsBOOL(Foundation.NSDictionary.writeToURL_atomically_, 1)

        self.assertArgIsSEL(
            Foundation.NSDictionary.keysSortedByValueUsingSelector_, 0, b"i@:@"
        )

        self.assertArgIsIn(
            Foundation.NSDictionary.dictionaryWithObjects_forKeys_count_, 0
        )
        self.assertArgSizeInArg(
            Foundation.NSDictionary.dictionaryWithObjects_forKeys_count_, 0, 2
        )
        self.assertArgIsIn(
            Foundation.NSDictionary.dictionaryWithObjects_forKeys_count_, 1
        )
        self.assertArgSizeInArg(
            Foundation.NSDictionary.dictionaryWithObjects_forKeys_count_, 1, 2
        )

        self.assertArgIsIn(Foundation.NSDictionary.initWithObjects_forKeys_count_, 0)
        self.assertArgSizeInArg(
            Foundation.NSDictionary.initWithObjects_forKeys_count_, 0, 2
        )
        self.assertArgIsIn(Foundation.NSDictionary.initWithObjects_forKeys_count_, 1)
        self.assertArgSizeInArg(
            Foundation.NSDictionary.initWithObjects_forKeys_count_, 1, 2
        )

        self.assertArgIsBOOL(Foundation.NSDictionary.initWithDictionary_copyItems_, 1)

        self.assertIsNullTerminated(Foundation.NSDictionary.initWithObjectsAndKeys_)
        self.assertIsNullTerminated(
            Foundation.NSDictionary.dictionaryWithObjectsAndKeys_
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            Foundation.NSDictionary.enumerateKeysAndObjectsUsingBlock_,
            0,
            b"v@@o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSDictionary.enumerateKeysAndObjectsWithOptions_usingBlock_,
            1,
            b"v@@o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSDictionary.keysSortedByValueUsingComparator_, 0, b"i@@"
        )
        self.assertArgIsBlock(
            Foundation.NSDictionary.keysSortedByValueWithOptions_usingComparator_,
            1,
            objc._C_NSInteger + b"@@",
        )

        self.assertArgIsBlock(
            Foundation.NSDictionary.keysOfEntriesPassingTest_,
            0,
            objc._C_NSBOOL + b"@@o^" + objc._C_NSBOOL,
        )
        self.assertArgIsBlock(
            Foundation.NSDictionary.keysOfEntriesWithOptions_passingTest_,
            1,
            objc._C_NSBOOL + b"@@o^" + objc._C_NSBOOL,
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsOut(Foundation.NSDictionary.writeToURL_error_, 1)
        self.assertResultIsBOOL(Foundation.NSDictionary.writeToURL_error_)

        self.assertArgIsOut(Foundation.NSDictionary.initWithContentsOfURL_error_, 1)
        self.assertArgIsOut(
            Foundation.NSDictionary.dictionaryWithContentsOfURL_error_, 1
        )



import Cocoa
import objc
from objc import super


class MyWindowController(Cocoa.NSWindowController):
    rectGradientView = objc.IBOutlet()
    bezierGradientView = objc.IBOutlet()

    startColorWell = objc.IBOutlet()
    endColorWell = objc.IBOutlet()
    angle = objc.IBOutlet()
    angleSlider = objc.IBOutlet()

    radialCheck = objc.IBOutlet()
    radialExplainText = objc.IBOutlet()

    def initWithPath_(self, newPath):
        return super().initWithWindowNibName_("TestWindow")

    def awakeFromNib(self):
        # make sure our angle text input keep the right format
        formatter = Cocoa.NSNumberFormatter.alloc().init()
        formatter.setNumberStyle_(Cocoa.NSNumberFormatterDecimalStyle)
        self.angle.cell().setFormatter_(formatter)

        # setup the initial start color
        self.rectGradientView.setStartColor_(Cocoa.NSColor.orangeColor())
        self.bezierGradientView.setStartColor_(Cocoa.NSColor.orangeColor())
        self.startColorWell.setColor_(Cocoa.NSColor.orangeColor())

        # setup the initial end color
        self.rectGradientView.setEndColor_(Cocoa.NSColor.blueColor())
        self.bezierGradientView.setEndColor_(Cocoa.NSColor.blueColor())
        self.endColorWell.setColor_(Cocoa.NSColor.blueColor())

        # setup the initial angle value
        self.rectGradientView.setAngle_(90.0)
        self.bezierGradientView.setAngle_(90.0)
        self.angle.setStringValue_("90.0")
        self.angleSlider.setFloatValue_(90.0)

    @objc.IBAction
    def swapColors_(self, sender):
        startColor = self.startColorWell.color()
        endColor = self.endColorWell.color()

        # change all our view's start and end colors
        self.rectGradientView.setStartColor_(endColor)
        self.rectGradientView.setEndColor_(startColor)

        self.bezierGradientView.setStartColor_(endColor)
        self.bezierGradientView.setEndColor_(startColor)

        # fix our color wells
        self.startColorWell.setColor_(endColor)
        self.endColorWell.setColor_(startColor)

    @objc.IBAction
    def startColor_(self, sender):
        newColor = sender.color()
        self.rectGradientView.setStartColor_(newColor)
        self.bezierGradientView.setStartColor_(newColor)

    @objc.IBAction
    def endColor_(self, sender):
        newColor = sender.color()
        self.rectGradientView.setEndColor_(newColor)
        self.bezierGradientView.setEndColor_(newColor)

    def controlTextDidEndEditing_(self, notification):
        theAngle = self.angle.floatValue()
        self.rectGradientView.setAngle_(theAngle)
        self.bezierGradientView.setAngle_(theAngle)

        theAngleDougle = self.angle.doubleValue()
        self.angleSlider.setDoubleValue_(theAngleDougle)
        self.angleSlider.setNeedsDisplay_(True)

    @objc.IBAction
    def angleSliderChange_(self, sender):
        angleValue = sender.floatValue()
        self.rectGradientView.setAngle_(angleValue)
        self.bezierGradientView.setAngle_(angleValue)
        self.angle.setDoubleValue_(angleValue)

    @objc.IBAction
    def radialDraw_(self, sender):
        self.rectGradientView.setRadialDraw_(sender.selectedCell().state())
        self.bezierGradientView.setRadialDraw_(sender.selectedCell().state())

        # angle factor does not relate to radial draws
        self.angleSlider.setEnabled_(not sender.selectedCell().state())
        self.angle.setEnabled_(not sender.selectedCell().state())

        # hide/show the explain text for radial gradients
        self.radialExplainText.setHidden_(not sender.selectedCell().state())



"""
Some simple tests to check that the framework is properly wrapped.
"""
import os

import Foundation
from PyObjCTools.TestSupport import TestCase


class TestFoundation(TestCase):
    def testValues(self):
        self.assertHasAttr(Foundation, "NSTimeIntervalSince1970")
        self.assertIsInstance(Foundation.NSTimeIntervalSince1970, float)
        self.assertEqual(Foundation.NSTimeIntervalSince1970, 978_307_200.0)

        if int(os.uname()[2].split(".")[0]) < 9:
            self.assertHasAttr(Foundation, "NSMaximumStringLength")
            self.assertIsInstance(Foundation.NSMaximumStringLength, int)
        self.assertHasAttr(Foundation, "NSURLResponseUnknownLength")
        self.assertIsInstance(Foundation.NSURLResponseUnknownLength, int)

    def testFunctions(self):
        self.assertHasAttr(Foundation, "NSStringFromSelector")

    def testProtocols(self):
        self.assertNotHasAttr(Foundation, "protocols")

    def test_structs(self):
        self.assertHasAttr(Foundation, "NSPoint")
        o = Foundation.NSPoint()
        self.assertHasAttr(o, "x")
        self.assertHasAttr(o, "y")

        self.assertHasAttr(Foundation, "NSSize")
        o = Foundation.NSSize()
        self.assertHasAttr(o, "width")
        self.assertHasAttr(o, "height")

        self.assertHasAttr(Foundation, "NSRange")
        o = Foundation.NSRange()
        self.assertHasAttr(o, "location")
        self.assertHasAttr(o, "length")

        self.assertHasAttr(Foundation, "NSRect")
        o = Foundation.NSRect()
        self.assertHasAttr(o, "origin")
        self.assertHasAttr(o, "size")
        self.assertIsInstance(o.origin, Foundation.NSPoint)
        self.assertIsInstance(o.size, Foundation.NSSize)

        self.assertHasAttr(Foundation, "NSAffineTransformStruct")
        o = Foundation.NSAffineTransformStruct()
        self.assertHasAttr(o, "m11")
        self.assertHasAttr(o, "m12")
        self.assertHasAttr(o, "m21")
        self.assertHasAttr(o, "m22")
        self.assertHasAttr(o, "tX")
        self.assertHasAttr(o, "tY")



import os

import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSLog(TestCase):
    def _redirect(self):
        fd = os.open("pyobjc-test-nslog.txt", os.O_RDWR | os.O_CREAT, 0o666)
        self.real_stderr = os.dup(2)
        os.dup2(fd, 2)
        os.close(fd)

    def _cleanup(self):
        os.dup2(self.real_stderr, 2)
        fp = open("pyobjc-test-nslog.txt", "rb")
        data = fp.read()
        fp.close()
        os.unlink("pyobjc-test-nslog.txt")
        return data

    def testBasic(self):
        self._redirect()
        try:
            Foundation.NSLog("Hello world")
        finally:
            data = self._cleanup().rstrip()

        self.assertTrue(data.endswith(b"] Hello world"))

    def testWithArguments(self):
        self._redirect()
        try:
            Foundation.NSLog("Hello %@: the count is %d", "ronald", 99)
        finally:
            data = self._cleanup().rstrip()

        self.assertTrue(data.endswith(b"] Hello ronald: the count is 99"))

    def testWithInvalidFormat(self):
        self._redirect()
        try:
            self.assertRaises(
                ValueError, Foundation.NSLog, "Hello %@: the count is %d", "ronald"
            )
            self.assertRaises(
                ValueError,
                Foundation.NSLog,
                "Hello %@: the count is %d",
                "ronald",
                "foo",
            )
            self.assertRaises(
                ValueError,
                Foundation.NSLog,
                "Hello %@: the count is %d",
                "ronald",
                42,
                "foo",
            )

        finally:
            self._cleanup()


class TestNSLogv(TestCase):
    def testNotSuchThing(self):
        self.assertFalse(hasattr(Foundation, "NSLogv"))



import gc

import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class PythonClass:
    def __init__(self):
        self.fireCount = 0

    def fire_(self, timer):
        self.fireCount += 1


class TestNSTimer(TestCase):
    def _testHelp(self):
        obj = PythonClass()
        pool = Foundation.NSAutoreleasePool.alloc().init()
        self.assertArgIsBOOL(
            Foundation.NSTimer.timerWithTimeInterval_target_selector_userInfo_repeats_,
            4,
        )
        timer = (
            Foundation.NSTimer.timerWithTimeInterval_target_selector_userInfo_repeats_(
                0.1, obj, "fire:", None, False
            )
        )
        Foundation.NSRunLoop.currentRunLoop().addTimer_forMode_(
            timer, Foundation.NSDefaultRunLoopMode
        )
        Foundation.NSRunLoop.currentRunLoop().runUntilDate_(
            Foundation.NSDate.dateWithTimeIntervalSinceNow_(0.5)
        )
        timer.invalidate()
        self.assertEqual(obj.fireCount, 1)

        del timer
        del pool

    def testPythonLeakage(self):

        # Ignore first run, this has some side-effects that would
        # taint the result.
        self._testHelp()

        # Now run the test again in a loop to detect leakage
        gc.collect()
        before = len(gc.get_objects())

        for _ in range(10):
            self._testHelp()

        gc.collect()
        after = len(gc.get_objects())

        for _ in range(10):
            self._testHelp()

        gc.collect()
        after2 = len(gc.get_objects())

        self.assertEqual(after, before, "%d - %d - %d" % (before, after, after2))

    def testMethods(self):
        self.assertArgIsBOOL(
            Foundation.NSTimer.timerWithTimeInterval_invocation_repeats_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSTimer.scheduledTimerWithTimeInterval_invocation_repeats_, 2
        )
        self.assertArgIsBOOL(
            Foundation.NSTimer.timerWithTimeInterval_target_selector_userInfo_repeats_,
            4,
        )
        self.assertArgIsSEL(
            Foundation.NSTimer.timerWithTimeInterval_target_selector_userInfo_repeats_,
            2,
            b"v@:@",
        )
        self.assertArgIsBOOL(
            Foundation.NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_,
            4,
        )
        self.assertArgIsSEL(
            Foundation.NSTimer.scheduledTimerWithTimeInterval_target_selector_userInfo_repeats_,
            2,
            b"v@:@",
        )

        self.assertArgIsBOOL(
            Foundation.NSTimer.initWithFireDate_interval_target_selector_userInfo_repeats_,
            5,
        )
        self.assertArgIsSEL(
            Foundation.NSTimer.initWithFireDate_interval_target_selector_userInfo_repeats_,
            3,
            b"v@:@",
        )

        self.assertResultIsBOOL(Foundation.NSTimer.isValid)

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertArgIsBOOL(Foundation.NSTimer.timerWithTimeInterval_repeats_block_, 1)
        self.assertArgIsBlock(
            Foundation.NSTimer.timerWithTimeInterval_repeats_block_, 2, b"v@"
        )

        self.assertArgIsBOOL(
            Foundation.NSTimer.scheduledTimerWithTimeInterval_repeats_block_, 1
        )
        self.assertArgIsBlock(
            Foundation.NSTimer.scheduledTimerWithTimeInterval_repeats_block_, 2, b"v@"
        )

        self.assertArgIsBOOL(
            Foundation.NSTimer.initWithFireDate_interval_repeats_block_, 2
        )
        self.assertArgIsBlock(
            Foundation.NSTimer.initWithFireDate_interval_repeats_block_, 3, b"v@"
        )



# This just tests the definitions in the Foundation.NSKeyedArchiver header
import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSKeyedArchiver(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSInvalidArchiveOperationException, str)
        self.assertIsInstance(Foundation.NSInvalidUnarchiveOperationException, str)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(Foundation.NSKeyedArchiveRootObjectKey, str)

    def testOutput(self):
        o = Foundation.NSKeyedUnarchiver.alloc().initForReadingWithData_(
            Foundation.NSKeyedArchiver.archivedDataWithRootObject_("foobar")
        )
        self.assertIsInstance(o, Foundation.NSKeyedUnarchiver)
        m = o.decodeBytesForKey_returnedLength_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"^v")
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

        data = Foundation.NSMutableData.alloc().init()
        o = Foundation.NSArchiver.alloc().initForWritingWithMutableData_(data)
        m = o.encodeBytes_length_forKey_.__metadata__()
        self.assertEqual(m["arguments"][2]["type"], b"n^v")

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSKeyedArchiver.archiveRootObject_toFile_)
        self.assertArgIsBOOL(Foundation.NSKeyedArchiver.encodeBool_forKey_, 0)
        self.assertArgHasType(
            Foundation.NSKeyedArchiver.encodeBytes_length_forKey_, 0, b"n^v"
        )
        self.assertArgSizeInArg(
            Foundation.NSKeyedArchiver.encodeBytes_length_forKey_, 0, 1
        )

        self.assertResultIsBOOL(Foundation.NSKeyedUnarchiver.containsValueForKey_)
        self.assertResultIsBOOL(Foundation.NSKeyedUnarchiver.decodeBoolForKey_)

        self.assertResultHasType(
            Foundation.NSKeyedUnarchiver.decodeBytesForKey_returnedLength_, b"^v"
        )
        self.assertResultSizeInArg(
            Foundation.NSKeyedUnarchiver.decodeBytesForKey_returnedLength_, 1
        )
        self.assertArgIsOut(
            Foundation.NSKeyedUnarchiver.decodeBytesForKey_returnedLength_, 1
        )

        self.assertArgHasType(
            Foundation.NSCoder.encodePoint_forKey_, 0, Foundation.NSPoint.__typestr__
        )
        self.assertArgHasType(
            Foundation.NSCoder.encodeSize_forKey_, 0, Foundation.NSSize.__typestr__
        )
        self.assertArgHasType(
            Foundation.NSCoder.encodeRect_forKey_, 0, Foundation.NSRect.__typestr__
        )
        self.assertResultHasType(
            Foundation.NSCoder.decodePointForKey_, Foundation.NSPoint.__typestr__
        )
        self.assertResultHasType(
            Foundation.NSCoder.decodeSizeForKey_, Foundation.NSSize.__typestr__
        )
        self.assertResultHasType(
            Foundation.NSCoder.decodeRectForKey_, Foundation.NSRect.__typestr__
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(Foundation.NSKeyedArchiver.requiresSecureCoding)
        self.assertArgIsBOOL(Foundation.NSKeyedArchiver.setRequiresSecureCoding_, 0)
        self.assertResultIsBOOL(Foundation.NSKeyedUnarchiver.requiresSecureCoding)
        self.assertArgIsBOOL(Foundation.NSKeyedUnarchiver.setRequiresSecureCoding_, 0)

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBOOL(Foundation.NSKeyedArchiver.initRequiringSecureCoding_, 0)
        self.assertArgIsBOOL(
            Foundation.NSKeyedArchiver.archivedDataWithRootObject_requiringSecureCoding_error_,
            1,
        )
        self.assertArgIsOut(
            Foundation.NSKeyedArchiver.archivedDataWithRootObject_requiringSecureCoding_error_,
            2,
        )

        self.assertArgIsOut(
            Foundation.NSKeyedUnarchiver.initForReadingFromData_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSKeyedUnarchiver.unarchivedObjectOfClass_fromData_error_, 2
        )
        self.assertArgIsOut(
            Foundation.NSKeyedUnarchiver.unarchivedObjectOfClasses_fromData_error_, 2
        )

    @min_os_level("11.0")
    def testMethods11_0(self):
        self.assertArgIsOut(
            Foundation.NSKeyedUnarchiver.unarchivedArrayOfObjectsOfClasses_fromData_error_,
            2,
        )
        self.assertArgIsOut(
            Foundation.NSKeyedUnarchiver.unarchivedDictionaryWithKeysOfClasses_objectsOfClasses_fromData_error_,
            3,
        )

    @min_sdk_level("10.7")
    def testProtocols(self):
        objc.protocolNamed("NSKeyedArchiverDelegate")
        objc.protocolNamed("NSKeyedUnarchiverDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSSavePanelHelper(AppKit.NSObject):
    def panel_shouldShowFilename_(self, p, f):
        return 1

    def panel_compareFilename_with_caseSensitive_(self, p, f1, f2, i):
        return 1

    def panel_isValidFilename_(self, p, f):
        return 1

    def panel_userEnteredFilename_confirmed_(self, p, f, c):
        return 1

    def panel_willExpand_(self, s, e):
        return 1

    def panel_directoryDidChange_(self, s, p):
        pass

    def panelSelectionDidChange_(self, s):
        pass

    def panel_shouldEnableURL_(self, p, u):
        return 1

    def panel_validateURL_error_(self, p, u, e):
        return 1


class TestNSSavePanel(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSFileHandlingPanelCancelButton, AppKit.NSCancelButton)
        self.assertEqual(AppKit.NSFileHandlingPanelOKButton, AppKit.NSOKButton)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSavePanel.showsHiddenFiles)
        self.assertArgIsBOOL(AppKit.NSSavePanel.setShowsHiddenFiles_, 0)
        self.assertResultIsBOOL(AppKit.NSSavePanel.allowsOtherFileTypes)
        self.assertArgIsBOOL(AppKit.NSSavePanel.setAllowsOtherFileTypes_, 0)
        self.assertResultIsBOOL(AppKit.NSSavePanel.isExpanded)
        self.assertResultIsBOOL(AppKit.NSSavePanel.canCreateDirectories)
        self.assertArgIsBOOL(AppKit.NSSavePanel.setCanCreateDirectories_, 0)
        self.assertResultIsBOOL(AppKit.NSSavePanel.canSelectHiddenExtension)
        self.assertArgIsBOOL(AppKit.NSSavePanel.setCanSelectHiddenExtension_, 0)
        self.assertResultIsBOOL(AppKit.NSSavePanel.isExtensionHidden)
        self.assertArgIsBOOL(AppKit.NSSavePanel.setExtensionHidden_, 0)
        self.assertResultIsBOOL(AppKit.NSSavePanel.treatsFilePackagesAsDirectories)
        self.assertArgIsBOOL(AppKit.NSSavePanel.setTreatsFilePackagesAsDirectories_, 0)

        self.assertArgIsSEL(
            AppKit.NSSavePanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo_,  # noqa: B950
            4,
            b"v@:@" + objc._C_NSInteger + b"^v",
        )
        self.assertArgHasType(
            AppKit.NSSavePanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo_,  # noqa: B950
            5,
            b"^v",
        )

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSOpenSavePanelDelegate")

    def testProtocol(self):
        self.assertResultIsBOOL(TestNSSavePanelHelper.panel_shouldShowFilename_)
        self.assertResultHasType(
            TestNSSavePanelHelper.panel_compareFilename_with_caseSensitive_,
            objc._C_NSInteger,
        )
        self.assertArgIsBOOL(
            TestNSSavePanelHelper.panel_compareFilename_with_caseSensitive_, 3
        )
        self.assertResultIsBOOL(TestNSSavePanelHelper.panel_isValidFilename_)
        self.assertArgIsBOOL(
            TestNSSavePanelHelper.panel_userEnteredFilename_confirmed_, 2
        )
        self.assertArgIsBOOL(TestNSSavePanelHelper.panel_willExpand_, 1)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            AppKit.NSSavePanel.beginWithCompletionHandler_, 0, b"v" + objc._C_NSInteger
        )

        self.assertResultIsBOOL(TestNSSavePanelHelper.panel_shouldEnableURL_)
        self.assertResultIsBOOL(TestNSSavePanelHelper.panel_validateURL_error_)
        self.assertArgHasType(TestNSSavePanelHelper.panel_validateURL_error_, 2, b"o^@")

        self.assertArgIsBlock(
            AppKit.NSSavePanel.beginSheetModalForWindow_completionHandler_,
            1,
            b"v" + objc._C_NSInteger,
        )
        self.assertArgIsBlock(
            AppKit.NSSavePanel.beginWithCompletionHandler_, 0, b"v" + objc._C_NSInteger
        )

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBOOL(AppKit.NSSavePanel.setShowsTagField_, 0)
        self.assertResultIsBOOL(AppKit.NSSavePanel.showsTagField)

    def test_issue282(self):
        panel = AppKit.NSSavePanel.savePanel()
        self.assertIsInstance(panel, AppKit.NSSavePanel)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, max_os_level


class TestNSOpenGL(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSOpenGLGOFormatCacheSize, 501)
        self.assertEqual(AppKit.NSOpenGLGOClearFormatCache, 502)
        self.assertEqual(AppKit.NSOpenGLGORetainRenderers, 503)
        self.assertEqual(AppKit.NSOpenGLGOResetLibrary, 504)
        self.assertEqual(AppKit.NSOpenGLGOUseBuildCache, 506)

        self.assertEqual(AppKit.NSOpenGLPFAAllRenderers, 1)
        self.assertEqual(AppKit.NSOpenGLPFATripleBuffer, 3)
        self.assertEqual(AppKit.NSOpenGLPFADoubleBuffer, 5)
        self.assertEqual(AppKit.NSOpenGLPFAStereo, 6)
        self.assertEqual(AppKit.NSOpenGLPFAAuxBuffers, 7)
        self.assertEqual(AppKit.NSOpenGLPFAColorSize, 8)
        self.assertEqual(AppKit.NSOpenGLPFAAlphaSize, 11)
        self.assertEqual(AppKit.NSOpenGLPFADepthSize, 12)
        self.assertEqual(AppKit.NSOpenGLPFAStencilSize, 13)
        self.assertEqual(AppKit.NSOpenGLPFAAccumSize, 14)
        self.assertEqual(AppKit.NSOpenGLPFAMinimumPolicy, 51)
        self.assertEqual(AppKit.NSOpenGLPFAMaximumPolicy, 52)
        self.assertEqual(AppKit.NSOpenGLPFAOffScreen, 53)
        self.assertEqual(AppKit.NSOpenGLPFAFullScreen, 54)
        self.assertEqual(AppKit.NSOpenGLPFASampleBuffers, 55)
        self.assertEqual(AppKit.NSOpenGLPFASamples, 56)
        self.assertEqual(AppKit.NSOpenGLPFAAuxDepthStencil, 57)
        self.assertEqual(AppKit.NSOpenGLPFAColorFloat, 58)
        self.assertEqual(AppKit.NSOpenGLPFAMultisample, 59)
        self.assertEqual(AppKit.NSOpenGLPFASupersample, 60)
        self.assertEqual(AppKit.NSOpenGLPFASampleAlpha, 61)
        self.assertEqual(AppKit.NSOpenGLPFARendererID, 70)
        self.assertEqual(AppKit.NSOpenGLPFASingleRenderer, 71)
        self.assertEqual(AppKit.NSOpenGLPFANoRecovery, 72)
        self.assertEqual(AppKit.NSOpenGLPFAAccelerated, 73)
        self.assertEqual(AppKit.NSOpenGLPFAClosestPolicy, 74)
        self.assertEqual(AppKit.NSOpenGLPFARobust, 75)
        self.assertEqual(AppKit.NSOpenGLPFABackingStore, 76)
        self.assertEqual(AppKit.NSOpenGLPFAMPSafe, 78)
        self.assertEqual(AppKit.NSOpenGLPFAWindow, 80)
        self.assertEqual(AppKit.NSOpenGLPFAMultiScreen, 81)
        self.assertEqual(AppKit.NSOpenGLPFACompliant, 83)
        self.assertEqual(AppKit.NSOpenGLPFAScreenMask, 84)
        self.assertEqual(AppKit.NSOpenGLPFAPixelBuffer, 90)
        self.assertEqual(AppKit.NSOpenGLPFAVirtualScreenCount, 128)

        self.assertEqual(AppKit.NSOpenGLCPSwapRectangle, 200)
        self.assertEqual(AppKit.NSOpenGLCPSwapRectangleEnable, 201)
        self.assertEqual(AppKit.NSOpenGLCPRasterizationEnable, 221)
        self.assertEqual(AppKit.NSOpenGLCPSwapInterval, 222)
        self.assertEqual(AppKit.NSOpenGLCPSurfaceOrder, 235)
        self.assertEqual(AppKit.NSOpenGLCPSurfaceOpacity, 236)
        self.assertEqual(AppKit.NSOpenGLCPStateValidation, 301)
        self.assertEqual(AppKit.NSOpenGLCPSurfaceBackingSize, 304)
        self.assertEqual(AppKit.NSOpenGLCPReclaimResources, 308)
        self.assertEqual(AppKit.NSOpenGLCPCurrentRendererID, 309)
        self.assertEqual(AppKit.NSOpenGLCPGPUVertexProcessing, 310)
        self.assertEqual(AppKit.NSOpenGLCPGPUFragmentProcessing, 311)
        self.assertEqual(AppKit.NSOpenGLCPHasDrawable, 314)
        self.assertEqual(AppKit.NSOpenGLCPMPSwapsInFlight, 315)
        self.assertEqual(AppKit.NSOpenGLCPSurfaceSurfaceVolatile, 306)

        self.assertEqual(AppKit.NSOpenGLContextParameterSwapInterval, 222)
        self.assertEqual(AppKit.NSOpenGLContextParameterSurfaceOrder, 235)
        self.assertEqual(AppKit.NSOpenGLContextParameterSurfaceOpacity, 236)
        self.assertEqual(AppKit.NSOpenGLContextParameterSurfaceBackingSize, 304)
        self.assertEqual(AppKit.NSOpenGLContextParameterReclaimResources, 308)
        self.assertEqual(AppKit.NSOpenGLContextParameterCurrentRendererID, 309)
        self.assertEqual(AppKit.NSOpenGLContextParameterGPUVertexProcessing, 310)
        self.assertEqual(AppKit.NSOpenGLContextParameterGPUFragmentProcessing, 311)
        self.assertEqual(AppKit.NSOpenGLContextParameterHasDrawable, 314)
        self.assertEqual(AppKit.NSOpenGLContextParameterMPSwapsInFlight, 315)
        self.assertEqual(AppKit.NSOpenGLContextParameterSwapRectangle, 200)
        self.assertEqual(AppKit.NSOpenGLContextParameterSwapRectangleEnable, 201)
        self.assertEqual(AppKit.NSOpenGLContextParameterRasterizationEnable, 221)
        self.assertEqual(AppKit.NSOpenGLContextParameterStateValidation, 301)
        self.assertEqual(AppKit.NSOpenGLContextParameterSurfaceSurfaceVolatile, 306)

    @min_os_level("10.5")
    def testConstants10_5(self):
        self.assertEqual(AppKit.NSOpenGLPFAAllowOfflineRenderers, 96)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSOpenGLPFAOpenGLProfile, 99)
        self.assertEqual(AppKit.NSOpenGLProfileVersionLegacy, 0x1000)
        self.assertEqual(AppKit.NSOpenGLProfileVersion3_2Core, 0x3200)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(AppKit.NSOpenGLProfileVersion4_1Core, 0x4100)

    def testFunctions(self):
        major, minor = AppKit.NSOpenGLGetVersion(None, None)
        self.assertIsInstance(major, int)
        self.assertIsInstance(minor, int)

        self.assertArgIsOut(AppKit.NSOpenGLGetOption, 1)
        v = AppKit.NSOpenGLGetOption(AppKit.NSOpenGLGOFormatCacheSize, None)
        self.assertIsInstance(v, int)

        AppKit.NSOpenGLSetOption(AppKit.NSOpenGLGOFormatCacheSize, v)

    def testMethods(self):
        self.assertArgIsNullTerminated(
            AppKit.NSOpenGLPixelFormat.initWithAttributes_, 0
        )
        self.assertArgIsIn(AppKit.NSOpenGLPixelFormat.initWithAttributes_, 0)

        o = AppKit.NSOpenGLPixelFormat.alloc().initWithAttributes_(
            [AppKit.NSOpenGLPFANoRecovery, AppKit.NSOpenGLPFAAuxBuffers, 2]
        )
        self.assertIsInstance(o, AppKit.NSOpenGLPixelFormat)

        # FIXME: I'm not entirely sure this test is correct.
        self.assertArgIsOut(
            AppKit.NSOpenGLPixelFormat.getValues_forAttribute_forVirtualScreen_, 0
        )
        v = o.getValues_forAttribute_forVirtualScreen_(
            None, AppKit.NSOpenGLPFANoRecovery, 0
        )
        self.assertIsInstance(v, int)

        self.assertResultHasType(
            AppKit.NSOpenGLPixelFormat.CGLPixelFormatObj, b"^{_CGLPixelFormatObject}"
        )

        self.assertResultHasType(
            AppKit.NSOpenGLContext.CGLContextObj, b"^{_CGLContextObj}"
        )

        self.assertArgIsIn(AppKit.NSOpenGLContext.setValues_forParameter_, 0)
        self.assertArgIsVariableSize(AppKit.NSOpenGLContext.setValues_forParameter_, 0)

        self.assertArgIsIn(AppKit.NSOpenGLContext.setValues_forParameter_, 0)
        self.assertArgIsVariableSize(AppKit.NSOpenGLContext.setValues_forParameter_, 0)

        self.assertArgIsIn(
            AppKit.NSOpenGLContext.setOffScreen_width_height_rowbytes_, 0
        )
        self.assertArgIsVariableSize(
            AppKit.NSOpenGLContext.setOffScreen_width_height_rowbytes_, 0
        )

    @min_os_level("10.6")
    @max_os_level("10.11")
    def testMethods10_6(self):
        self.assertArgHasType(
            AppKit.NSOpenGLPixelFormat.initWithCGLPixelFormatObj_,
            0,
            b"^{_CGLPixelFormatObject=}",
        )
        self.assertArgHasType(
            AppKit.NSOpenGLPixelFormat.initWithCGLBufferObj_, 0, b"^{_CGLBufferObject=}"
        )
        self.assertResultHasType(
            AppKit.NSOpenGLPixelFormat.CGLBufferObj_, b"^{_CGLBufferObject=}"
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(AppKit.NSOpenGLPFARemotePixelBuffer, 91)
        self.assertEqual(AppKit.NSOpenGLPFAAcceleratedCompute, 97)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestNSHashTable(TestCase):
    def testConvenience(self):
        v = Foundation.NSHashTable.hashTableWithOptions_(
            Foundation.NSPointerFunctionsObjectPersonality
        )
        self.assertEqual(len(v), 0)

        v.add(32)
        v.add(2)

        self.assertIn(32, v)
        self.assertNotIn(42, v)

        self.assertNotIn(1, v)
        self.assertIn(2, v)
        v.remove(2)
        v.remove(1)
        self.assertNotIn(1, v)
        self.assertNotIn(2, v)

        v.clear()
        self.assertEqual(len(v), 0)

        v.add(1)
        v.add(2)
        v.add(3)

        self.assertEqual(len(v), 3)
        self.assertIn(v.pop(), {1, 2, 3})
        self.assertIn(v.pop(), {1, 2, 3})
        self.assertIn(v.pop(), {1, 2, 3})
        self.assertEqual(len(v), 0)

        v.add(None)
        self.assertEqual(len(v), 1)
        self.assertIn(None, v)
        v.remove(None)
        self.assertEqual(len(v), 0)

        v.add(None)
        o = v.pop()
        self.assertEqual(o, None)
        self.assertEqual(len(v), 0)

    def testConstants(self):
        self.assertEqual(Foundation.NSHashTableStrongMemory, 0)
        self.assertEqual(
            Foundation.NSHashTableZeroingWeakMemory,
            Foundation.NSPointerFunctionsZeroingWeakMemory,
        )
        self.assertEqual(
            Foundation.NSHashTableCopyIn, Foundation.NSPointerFunctionsCopyIn
        )
        self.assertEqual(
            Foundation.NSHashTableObjectPointerPersonality,
            Foundation.NSPointerFunctionsObjectPointerPersonality,
        )

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(
            Foundation.NSHashTableWeakMemory, Foundation.NSPointerFunctionsWeakMemory
        )

    @expectedFailure
    def testFunctions(self):
        self.fail("NSHasTable functions")

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSHashTable.containsObject_)
        self.assertResultIsBOOL(Foundation.NSHashTable.intersectsHashTable_)
        self.assertResultIsBOOL(Foundation.NSHashTable.isEqualToHashTable_)
        self.assertResultIsBOOL(Foundation.NSHashTable.isSubsetOfHashTable_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSStackView(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSUserInterfaceLayoutOrientationHorizontal, 0)
        self.assertEqual(AppKit.NSUserInterfaceLayoutOrientationVertical, 1)

        self.assertEqual(AppKit.NSStackViewGravityTop, 1)
        self.assertEqual(AppKit.NSStackViewGravityLeading, 1)
        self.assertEqual(AppKit.NSStackViewGravityCenter, 2)
        self.assertEqual(AppKit.NSStackViewGravityBottom, 3)
        self.assertEqual(AppKit.NSStackViewGravityTrailing, 3)

        self.assertEqual(AppKit.NSStackViewVisibilityPriorityMustHold, 1000.0)
        self.assertEqual(
            AppKit.NSStackViewVisibilityPriorityDetachOnlyIfNecessary, 900.0
        )
        self.assertEqual(AppKit.NSStackViewVisibilityPriorityNotVisible, 0.0)

        self.assertIsInstance(AppKit.NSStackViewSpacingUseDefault, float)
        self.assertEqual(AppKit.NSStackViewSpacingUseDefault, objc._FLT_MAX)

        self.assertEqual(AppKit.NSStackViewDistributionGravityAreas, -1)
        self.assertEqual(AppKit.NSStackViewDistributionFill, 0)
        self.assertEqual(AppKit.NSStackViewDistributionFillEqually, 1)
        self.assertEqual(AppKit.NSStackViewDistributionFillProportionally, 2)
        self.assertEqual(AppKit.NSStackViewDistributionEqualSpacing, 3)
        self.assertEqual(AppKit.NSStackViewDistributionEqualCentering, 4)

    @min_os_level("10.9")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSStackView.hasEqualSpacing)
        self.assertArgIsBOOL(AppKit.NSStackView.setHasEqualSpacing_, 0)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertResultIsBOOL(AppKit.NSStackView.detachesHiddenViews)
        self.assertArgIsBOOL(AppKit.NSStackView.setDetachesHiddenViews_, 0)

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSStackViewDelegate")



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestValueTrans(TestCase):
    def testConstants(self):
        self.assertIsInstance(Foundation.NSNegateBooleanTransformerName, str)
        self.assertIsInstance(Foundation.NSIsNilTransformerName, str)
        self.assertIsInstance(Foundation.NSIsNotNilTransformerName, str)
        self.assertIsInstance(Foundation.NSUnarchiveFromDataTransformerName, str)
        self.assertIsInstance(Foundation.NSKeyedUnarchiveFromDataTransformerName, str)

    @min_os_level("10.14")
    def testConstants10_14(self):
        self.assertIsInstance(Foundation.NSSecureUnarchiveFromDataTransformerName, str)

    def testMethods(self):
        self.assertResultIsBOOL(
            Foundation.NSValueTransformer.allowsReverseTransformation
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSTextLayoutManagerHelper(AppKit.NSObject):
    def textLayoutManager_shouldBreakLineBeforeLocation_hyphenating_(self, a, b, c):
        return 1


class TestNSTextLayoutManager(TestCase):
    def test_constants(self):
        self.assertEqual(AppKit.NSTextLayoutManagerSegmentTypeStandard, 0)
        self.assertEqual(AppKit.NSTextLayoutManagerSegmentTypeSelection, 1)
        self.assertEqual(AppKit.NSTextLayoutManagerSegmentTypeHighlight, 2)

        self.assertEqual(AppKit.NSTextLayoutManagerSegmentOptionsNone, 0)
        self.assertEqual(
            AppKit.NSTextLayoutManagerSegmentOptionsRangeNotRequired, 1 << 0
        )
        self.assertEqual(
            AppKit.NSTextLayoutManagerSegmentOptionsMiddleFragmentsExcluded, 1 << 1
        )
        self.assertEqual(
            AppKit.NSTextLayoutManagerSegmentOptionsHeadSegmentExtended, 1 << 2
        )
        self.assertEqual(
            AppKit.NSTextLayoutManagerSegmentOptionsTailSegmentExtended, 1 << 3
        )
        self.assertEqual(
            AppKit.NSTextLayoutManagerSegmentOptionsUpstreamAffinity, 1 << 4
        )

    @min_sdk_level("12.0")
    def test_protocols(self):
        objc.protocolNamed("NSTextLayoutManagerDelegate")

    def test_methods(self):
        self.assertResultIsBOOL(
            TestNSTextLayoutManagerHelper.textLayoutManager_shouldBreakLineBeforeLocation_hyphenating_
        )
        self.assertArgIsBOOL(
            TestNSTextLayoutManagerHelper.textLayoutManager_shouldBreakLineBeforeLocation_hyphenating_,
            2,
        )

    @min_os_level("12.0")
    def test_methods12_0(self):
        self.assertResultIsBOOL(AppKit.NSTextLayoutManager.usesFontLeading)
        self.assertArgIsBOOL(AppKit.NSTextLayoutManager.setUsesFontLeading_, 0)

        self.assertResultIsBOOL(
            AppKit.NSTextLayoutManager.limitsLayoutForSuspiciousContents
        )
        self.assertArgIsBOOL(
            AppKit.NSTextLayoutManager.setLimitsLayoutForSuspiciousContents_, 0
        )

        self.assertResultIsBOOL(AppKit.NSTextLayoutManager.usesHyphenation)
        self.assertArgIsBOOL(AppKit.NSTextLayoutManager.setUsesHyphenation_, 0)

        self.assertArgIsBlock(
            AppKit.NSTextLayoutManager.enumerateTextLayoutFragmentsFromLocation_options_usingBlock_,
            2,
            b"Z@",
        )

        self.assertArgIsBOOL(
            AppKit.NSTextLayoutManager.enumerateRenderingAttributesFromLocation_reverse_usingBlock_,
            1,
        )
        self.assertArgIsBlock(
            AppKit.NSTextLayoutManager.enumerateRenderingAttributesFromLocation_reverse_usingBlock_,
            2,
            b"Z@@@",
        )

        self.assertArgIsBlock(
            AppKit.NSTextLayoutManager.enumerateTextSegmentsInRange_type_options_usingBlock_,
            3,
            b"Z@" + AppKit.NSRect.__typestr__ + b"d@",
        )



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestRunLoop(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFRunLoopRef)
        self.assertIsCFType(CoreFoundation.CFRunLoopSourceRef)
        self.assertIsCFType(CoreFoundation.CFRunLoopObserverRef)

        try:
            if objc.lookUpClass("__NSCFTimer") is CoreFoundation.CFRunLoopTimerRef:
                return
        except objc.error:
            pass
        try:
            if objc.lookUpClass("NSCFTimer") is CoreFoundation.CFRunLoopTimerRef:
                return
        except objc.error:
            pass
        self.assertIsCFType(CoreFoundation.CFRunLoopTimerRef)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFRunLoopRunFinished, 1)
        self.assertEqual(CoreFoundation.kCFRunLoopRunStopped, 2)
        self.assertEqual(CoreFoundation.kCFRunLoopRunTimedOut, 3)
        self.assertEqual(CoreFoundation.kCFRunLoopRunHandledSource, 4)
        self.assertEqual(CoreFoundation.kCFRunLoopEntry, (1 << 0))
        self.assertEqual(CoreFoundation.kCFRunLoopBeforeTimers, (1 << 1))
        self.assertEqual(CoreFoundation.kCFRunLoopBeforeSources, (1 << 2))
        self.assertEqual(CoreFoundation.kCFRunLoopBeforeWaiting, (1 << 5))
        self.assertEqual(CoreFoundation.kCFRunLoopAfterWaiting, (1 << 6))
        self.assertEqual(CoreFoundation.kCFRunLoopExit, (1 << 7))
        self.assertEqual(CoreFoundation.kCFRunLoopAllActivities, 0x0FFFFFFF)
        self.assertIsInstance(CoreFoundation.kCFRunLoopDefaultMode, str)
        self.assertIsInstance(CoreFoundation.kCFRunLoopCommonModes, str)

    def testGetTypeID(self):
        self.assertIsInstance(CoreFoundation.CFRunLoopGetTypeID(), int)
        self.assertIsInstance(CoreFoundation.CFRunLoopSourceGetTypeID(), int)
        self.assertIsInstance(CoreFoundation.CFRunLoopObserverGetTypeID(), int)
        self.assertIsInstance(CoreFoundation.CFRunLoopTimerGetTypeID(), int)

    def testRunloop(self):
        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        runloop_mode = "pyobjctest.cfrunloop"

        loop = CoreFoundation.CFRunLoopGetCurrent()
        self.assertIsInstance(loop, CoreFoundation.CFRunLoopRef)
        loop = CoreFoundation.CFRunLoopGetMain()
        self.assertIsInstance(loop, CoreFoundation.CFRunLoopRef)
        mode = CoreFoundation.CFRunLoopCopyCurrentMode(loop)
        if mode is not None:
            self.assertIsInstance(mode, str)
        self.assertResultIsCFRetained(CoreFoundation.CFRunLoopCopyAllModes)
        allmodes = CoreFoundation.CFRunLoopCopyAllModes(loop)
        self.assertIsInstance(allmodes, CoreFoundation.CFArrayRef)
        self.assertNotEqual(len(allmodes), 0)
        for mode in allmodes:
            self.assertIsInstance(mode, str)
        CoreFoundation.CFRunLoopAddCommonMode(loop, "pyobjctest")
        allmodes = CoreFoundation.CFRunLoopCopyAllModes(loop)

        tm = CoreFoundation.CFRunLoopGetNextTimerFireDate(loop, runloop_mode)
        self.assertIsInstance(tm, float)
        b = CoreFoundation.CFRunLoopIsWaiting(loop)
        self.assertIsInstance(b, bool)
        CoreFoundation.CFRunLoopWakeUp(loop)
        CoreFoundation.CFRunLoopStop(loop)

        res = CoreFoundation.CFRunLoopRunInMode("mode", 2.0, True)
        self.assertIsInstance(res, int)
        self.assertEqual(res, CoreFoundation.kCFRunLoopRunFinished)

        # CoreFoundation.CFRunLoopRun is hard to test reliably
        self.assertHasAttr(CoreFoundation, "CFRunLoopRun")

    def testObserver(self):
        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        runloop_mode = "pyobjctest.cfrunloop"

        rl = CoreFoundation.CFRunLoopGetCurrent()

        data = {}
        state = []

        def callback(observer, activity, info):
            state.append((observer, activity, info))

        observer = CoreFoundation.CFRunLoopObserverCreate(
            None,
            CoreFoundation.kCFRunLoopEntry | CoreFoundation.kCFRunLoopExit,
            True,
            4,
            callback,
            data,
        )
        self.assertIsInstance(observer, CoreFoundation.CFRunLoopObserverRef)
        ctx = CoreFoundation.CFRunLoopObserverGetContext(observer, None)
        self.assertIs(ctx, data)
        self.assertIs(CoreFoundation.CFRunLoopObserverDoesRepeat(observer), True)
        self.assertEqual(CoreFoundation.CFRunLoopObserverGetOrder(observer), 4)
        self.assertIs(CoreFoundation.CFRunLoopObserverIsValid(observer), True)
        self.assertEqual(
            CoreFoundation.CFRunLoopObserverGetActivities(observer),
            CoreFoundation.kCFRunLoopEntry | CoreFoundation.kCFRunLoopExit,
        )
        CoreFoundation.CFRunLoopObserverInvalidate(observer)
        self.assertIs(CoreFoundation.CFRunLoopObserverIsValid(observer), False)
        ctx = CoreFoundation.CFRunLoopObserverGetContext(observer, None)
        self.assertIs(ctx, objc.NULL)
        observer = CoreFoundation.CFRunLoopObserverCreate(
            None,
            CoreFoundation.kCFRunLoopEntry | CoreFoundation.kCFRunLoopExit,
            True,
            4,
            callback,
            data,
        )
        self.assertIsInstance(observer, CoreFoundation.CFRunLoopObserverRef)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsObserver(rl, observer, runloop_mode), False
        )
        CoreFoundation.CFRunLoopAddObserver(rl, observer, runloop_mode)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsObserver(rl, observer, runloop_mode), True
        )

        # Use dummy stream to ensure that the runloop actually performs work
        strval = b"hello world"
        stream = CoreFoundation.CFReadStreamCreateWithBytesNoCopy(
            None, strval, len(strval), CoreFoundation.kCFAllocatorNull
        )
        self.assertIsInstance(stream, CoreFoundation.CFReadStreamRef)
        CoreFoundation.CFReadStreamScheduleWithRunLoop(stream, rl, runloop_mode)
        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 1.0, True)
        self.assertIsInstance(res, int)
        CoreFoundation.CFReadStreamUnscheduleFromRunLoop(stream, rl, runloop_mode)

        self.assertNotEqual(len(state), 0)
        for item in state:
            self.assertIs(item[0], observer)
            self.assertIn(
                item[1], (CoreFoundation.kCFRunLoopEntry, CoreFoundation.kCFRunLoopExit)
            )
            self.assertIs(item[2], data)
        CoreFoundation.CFRunLoopRemoveObserver(rl, observer, runloop_mode)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsObserver(rl, observer, runloop_mode), False
        )

    def testTimer(self):
        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        runloop_mode = "pyobjctest.cfrunloop"
        rl = CoreFoundation.CFRunLoopGetCurrent()

        state = []
        data = {}

        def callback(timer, info):
            state.append((timer, info))

        timer = CoreFoundation.CFRunLoopTimerCreate(None, 0, 0.5, 0, 0, callback, data)

        r = CoreFoundation.CFRunLoopTimerGetNextFireDate(timer)
        self.assertIsInstance(r, float)
        CoreFoundation.CFRunLoopTimerSetNextFireDate(timer, r + 2)
        r2 = CoreFoundation.CFRunLoopTimerGetNextFireDate(timer)
        self.assertEqual(int(r2), int(r + 2))

        r = CoreFoundation.CFRunLoopTimerGetInterval(timer)
        self.assertEqual(r, 0.5)

        self.assertIs(CoreFoundation.CFRunLoopTimerGetContext(timer, None), data)
        self.assertIs(CoreFoundation.CFRunLoopTimerDoesRepeat(timer), True)
        self.assertEqual(CoreFoundation.CFRunLoopTimerGetOrder(timer), 0)
        self.assertIs(CoreFoundation.CFRunLoopTimerIsValid(timer), True)
        CoreFoundation.CFRunLoopTimerInvalidate(timer)
        self.assertIs(CoreFoundation.CFRunLoopTimerIsValid(timer), False)
        self.assertIs(CoreFoundation.CFRunLoopTimerGetContext(timer, None), objc.NULL)
        timer = CoreFoundation.CFRunLoopTimerCreate(None, 0, 0.5, 0, 0, callback, data)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsTimer(rl, timer, runloop_mode), False
        )
        CoreFoundation.CFRunLoopAddTimer(rl, timer, runloop_mode)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsTimer(rl, timer, runloop_mode), True
        )
        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 1.3, True)
        self.assertIsInstance(res, int)

        CoreFoundation.CFRunLoopTimerInvalidate(timer)
        CoreFoundation.CFRunLoopRemoveTimer(rl, timer, runloop_mode)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsTimer(rl, timer, runloop_mode), False
        )
        self.assertFalse(len(state) < 3)
        for item in state:
            self.assertIs(item[0], timer)
            self.assertIs(item[1], data)

    def testSource(self):
        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        runloop_mode = "pyobjctest.cfrunloop"

        rl = CoreFoundation.CFRunLoopGetCurrent()

        state = []
        data = {}

        def schedule(info, rl, mode):
            state.append(["schedule", info, rl, mode])

        def cancel(info, rl, mode):
            state.append(["cancel", info, rl, mode])

        def perform(info):
            state.append(["perform", info])

        source = CoreFoundation.CFRunLoopSourceCreate(
            None, 55, (0, schedule, cancel, perform, data)
        )
        self.assertIsInstance(source, CoreFoundation.CFRunLoopSourceRef)
        ctx = CoreFoundation.CFRunLoopSourceGetContext(source, None)
        self.assertIsInstance(ctx, tuple)
        self.assertEqual(ctx[0], 0)
        self.assertEqual(ctx[1], schedule)
        self.assertEqual(ctx[2], cancel)
        self.assertEqual(ctx[3], perform)
        self.assertEqual(ctx[4], data)

        self.assertEqual(CoreFoundation.CFRunLoopSourceGetOrder(source), 55)
        self.assertIs(CoreFoundation.CFRunLoopSourceIsValid(source), True)
        CoreFoundation.CFRunLoopSourceInvalidate(source)
        self.assertIs(CoreFoundation.CFRunLoopSourceIsValid(source), False)
        source = CoreFoundation.CFRunLoopSourceCreate(
            None, 55, (0, schedule, cancel, perform, data)
        )
        self.assertIsInstance(source, CoreFoundation.CFRunLoopSourceRef)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsSource(rl, source, runloop_mode), False
        )
        CoreFoundation.CFRunLoopAddSource(rl, source, runloop_mode)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsSource(rl, source, runloop_mode), True
        )
        self.assertEqual(len(state), 1)
        self.assertEqual(state[0][0], "schedule")
        self.assertIs(state[0][1], data)
        self.assertIs(state[0][2], rl)
        self.assertEqual(state[0][3], runloop_mode)
        del state[:]

        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 0.5, True)
        self.assertIsInstance(res, int)
        # self.assertEqual(res, CoreFoundation.kCFRunLoopRunTimedOut)

        self.assertEqual(len(state), 0)

        CoreFoundation.CFRunLoopSourceSignal(source)

        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 0.5, True)
        self.assertIsInstance(res, int)
        self.assertEqual(res, CoreFoundation.kCFRunLoopRunHandledSource)

        self.assertEqual(len(state), 1)
        self.assertEqual(state[0][0], "perform")
        self.assertIs(state[0][1], data)
        del state[:]

        CoreFoundation.CFRunLoopRemoveSource(rl, source, runloop_mode)
        self.assertIs(
            CoreFoundation.CFRunLoopContainsSource(rl, source, runloop_mode), False
        )
        self.assertEqual(len(state), 1)
        self.assertEqual(state[0][0], "cancel")
        self.assertIs(state[0][1], data)
        self.assertIs(state[0][2], rl)
        self.assertEqual(state[0][3], runloop_mode)

    @min_os_level("10.6")
    def testFunctions10_6(self):
        self.assertArgIsBlock(CoreFoundation.CFRunLoopPerformBlock, 2, b"v")

        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        rl = CoreFoundation.CFRunLoopGetCurrent()

        lst = []

        def doit():
            lst.append(True)

        CoreFoundation.CFRunLoopPerformBlock(rl, runloop_mode, doit)
        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 0.5, True)
        self.assertIsInstance(res, int)

        self.assertEqual(lst, [True])

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertArgIsBOOL(CoreFoundation.CFRunLoopObserverCreateWithHandler, 2)
        self.assertArgIsBlock(
            CoreFoundation.CFRunLoopObserverCreateWithHandler,
            4,
            b"v^{__CFRunLoopObserver=}" + objc._C_NSUInteger,
        )

        lst = []

        def record(observer, activity):
            lst.append((observer, activity))

        ref = CoreFoundation.CFRunLoopObserverCreateWithHandler(
            None, CoreFoundation.kCFRunLoopAllActivities, False, 0, record
        )
        self.assertIsInstance(ref, CoreFoundation.CFRunLoopObserverRef)

        runloop_mode = CoreFoundation.kCFRunLoopDefaultMode
        rl = CoreFoundation.CFRunLoopGetCurrent()

        CoreFoundation.CFRunLoopAddObserver(rl, ref, runloop_mode)
        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 0.5, True)
        CoreFoundation.CFRunLoopRemoveObserver(rl, ref, runloop_mode)

        self.assertNotEqual(lst, [])
        for a, b in lst:
            self.assertEqual(a, ref)
            self.assertIsInstance(b, int)

        self.assertArgIsBlock(
            CoreFoundation.CFRunLoopTimerCreateWithHandler, 5, b"v^{__CFRunLoopTimer=}"
        )
        lst = []
        ref = CoreFoundation.CFRunLoopTimerCreateWithHandler(
            None,
            CoreFoundation.CFAbsoluteTimeGetCurrent() + 2.9,
            0.0,
            0,
            0,
            lambda x: lst.append(x),
        )
        self.assertIsInstance(ref, CoreFoundation.CFRunLoopTimerRef)

        CoreFoundation.CFRunLoopAddTimer(rl, ref, runloop_mode)
        res = CoreFoundation.CFRunLoopRunInMode(runloop_mode, 6.0, True)
        self.assertIsInstance(res, int)
        CoreFoundation.CFRunLoopRemoveTimer(rl, ref, runloop_mode)

        import __main__

        # XXX: See issue #11 in the pyobjc tracker
        if "setup" in __main__.__file__:
            return

        self.assertNotEqual(lst, [])
        for a in lst:
            self.assertEqual(a, ref)

    @min_os_level("10.9")
    def testFunctions10_9(self):
        lst = []
        ref = CoreFoundation.CFRunLoopTimerCreateWithHandler(
            None,
            CoreFoundation.CFAbsoluteTimeGetCurrent() + 2.9,
            0.0,
            0,
            0,
            lambda x: lst.append(x),
        )
        self.assertIsInstance(ref, CoreFoundation.CFRunLoopTimerRef)
        CoreFoundation.CFRunLoopTimerSetTolerance(ref, 5.0)
        v = CoreFoundation.CFRunLoopTimerGetTolerance(ref)
        self.assertIsInstance(v, float)



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSPortMessage(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSPortMessage.sendBeforeDate_)



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase


class TestCFBag(TestCase):
    def testCreation(self):
        bag = CoreFoundation.CFBagCreate(None, [1, 1, 2, 3, 4], 5)
        self.assertIsInstance(bag, CoreFoundation.CFBagRef)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, 1), 2)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, 3), 1)

        bag = CoreFoundation.CFBagCreateMutable(None, 0)
        self.assertIsInstance(bag, CoreFoundation.CFBagRef)
        CoreFoundation.CFBagAddValue(bag, 9)
        CoreFoundation.CFBagAddValue(bag, 8)
        CoreFoundation.CFBagAddValue(bag, 9)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, 9), 2)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, 8), 1)

    def testApplyFunction(self):
        items = []
        contexts = []

        bag = CoreFoundation.CFBagCreate(None, [1, 1, 2, 3, 4], 5)

        def func(item, context):
            items.append(item * item)
            contexts.append(context)

        CoreFoundation.CFBagApplyFunction(bag, func, 99)
        items.sort()
        self.assertEqual(items, [1, 1, 4, 9, 16])
        self.assertEqual(contexts, [99, 99, 99, 99, 99])

    def testTypeID(self):
        v = CoreFoundation.CFBagGetTypeID()
        self.assertIsInstance(v, int)

    def testCopy(self):
        bag = CoreFoundation.CFBagCreate(None, [1, 1, 2, 3, 4], 5)
        self.assertIsInstance(bag, CoreFoundation.CFBagRef)
        bag2 = CoreFoundation.CFBagCreateCopy(None, bag)
        self.assertIsInstance(bag2, CoreFoundation.CFBagRef)
        bag3 = CoreFoundation.CFBagCreateMutableCopy(None, 0, bag)
        self.assertIsInstance(bag3, CoreFoundation.CFBagRef)
        self.assertIsNot(bag3, bag)

    def testInspect(self):
        bag = CoreFoundation.CFBagCreate(
            None, ["Hello", 42, "World", 42, "a", "a", "a"], 7
        )
        self.assertIsInstance(bag, CoreFoundation.CFBagRef)
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 7)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, "Hello"), 1)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, 42), 2)
        self.assertEqual(CoreFoundation.CFBagGetCountOfValue(bag, "a"), 3)
        self.assertTrue(CoreFoundation.CFBagContainsValue(bag, "a"))
        self.assertFalse(CoreFoundation.CFBagContainsValue(bag, "b"))

        v = CoreFoundation.CFBagGetValue(bag, "b")
        self.assertIs(v, None)
        v = CoreFoundation.CFBagGetValue(bag, "a")
        self.assertEqual(v, "a")
        exists, value = CoreFoundation.CFBagGetValueIfPresent(bag, "a", None)
        self.assertTrue(exists)
        self.assertEqual(value, "a")
        exists, value = CoreFoundation.CFBagGetValueIfPresent(bag, "b", None)
        self.assertFalse(exists)
        self.assertIs(value, None)
        values = set(CoreFoundation.CFBagGetValues(bag))
        expected = {"Hello", 42, "World", 42, "a", "a", "a"}
        self.assertEqual(values, expected)

    def testMutation(self):
        bag = CoreFoundation.CFBagCreateMutable(None, 0)
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 0)
        CoreFoundation.CFBagAddValue(bag, "hello")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 1)
        CoreFoundation.CFBagAddValue(bag, "hello")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 2)
        CoreFoundation.CFBagReplaceValue(bag, "hello")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 2)
        CoreFoundation.CFBagReplaceValue(bag, "world")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 2)
        CoreFoundation.CFBagSetValue(bag, "world")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 3)
        CoreFoundation.CFBagSetValue(bag, "world")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 3)
        CoreFoundation.CFBagRemoveValue(bag, "hello")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 2)
        CoreFoundation.CFBagRemoveValue(bag, "hello")
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 1)
        CoreFoundation.CFBagRemoveAllValues(bag)
        self.assertEqual(CoreFoundation.CFBagGetCount(bag), 0)

    def testFunctions(self):
        self.assertArgHasType(CoreFoundation.CFBagGetCountOfValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBagContainsValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBagGetValue, 1, b"@")
        self.assertResultHasType(CoreFoundation.CFBagGetValue, b"@")
        self.assertArgHasType(CoreFoundation.CFBagGetValueIfPresent, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBagGetValueIfPresent, 2, b"o^@")
        self.assertResultHasType(CoreFoundation.CFBagGetValueIfPresent, objc._C_NSBOOL)
        self.assertArgIsFunction(CoreFoundation.CFBagApplyFunction, 1, b"v@@", False)
        self.assertArgHasType(CoreFoundation.CFBagApplyFunction, 2, b"@")
        self.assertArgHasType(CoreFoundation.CFBagAddValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBagReplaceValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBagSetValue, 1, b"@")
        self.assertArgHasType(CoreFoundation.CFBagRemoveValue, 1, b"@")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSInvocation(TestCase):
    def test_dummy(self):
        value = Foundation.NSMutableArray.arrayWithArray_([1, 2, 3])

        invocation = Foundation.NSInvocation.invocationWithMethodSignature_(
            value.methodSignatureForSelector_("count")
        )
        invocation.setSelector_("count")
        invocation.setTarget_(value)
        invocation.invoke()

        v = invocation.getReturnValue_(None)
        self.assertIsInstance(v, int)
        self.assertEqual(v, 3)

        invocation = Foundation.NSInvocation.invocationWithMethodSignature_(
            value.methodSignatureForSelector_("addObject:")
        )
        invocation.setSelector_("addObject:")
        invocation.setTarget_(value)
        invocation.setArgument_atIndex_("hello", 2)
        v = invocation.getArgument_atIndex_(None, 2)
        self.assertEqual(v, "hello")
        invocation.invoke()

        self.assertEqual(value.count(), 4)

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSInvocation.argumentsRetained)

    def testNoUnsupported(self):
        self.assertNotHasAttr(Foundation, "NSObjCValue")
        self.assertNotHasAttr(Foundation, "NSObjCNoType")
        self.assertNotHasAttr(Foundation, "NSObjCVoidType")
        self.assertNotHasAttr(Foundation, "NSObjCCharType")
        self.assertNotHasAttr(Foundation, "NSObjCShortType")
        self.assertNotHasAttr(Foundation, "NSObjCLongType")
        self.assertNotHasAttr(Foundation, "NSObjCLonglongType")
        self.assertNotHasAttr(Foundation, "NSObjCFloatType")
        self.assertNotHasAttr(Foundation, "NSObjCDoubleType")
        self.assertNotHasAttr(Foundation, "NSObjCBoolType")
        self.assertNotHasAttr(Foundation, "NSObjCSelectorType")
        self.assertNotHasAttr(Foundation, "NSObjCObjectType")
        self.assertNotHasAttr(Foundation, "NSObjCStructType")
        self.assertNotHasAttr(Foundation, "NSObjCPointerType")
        self.assertNotHasAttr(Foundation, "NSObjCStringType")
        self.assertNotHasAttr(Foundation, "NSObjCArrayType")
        self.assertNotHasAttr(Foundation, "NSObjCUnionType")
        self.assertNotHasAttr(Foundation, "NSObjCBitfield")



#
#  ToolbarCreator.py
#  TableModelWithSearch
#
#  Created by Bill Bumgarner on Sun Apr 04 2004.
#  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
#

import objc
from Cocoa import (
    NSMenu,
    NSMenuItem,
    NSObject,
    NSToolbar,
    NSToolbarCustomizeToolbarItemIdentifier,
    NSToolbarFlexibleSpaceItemIdentifier,
    NSToolbarItem,
    NSToolbarPrintItemIdentifier,
    NSToolbarSeparatorItemIdentifier,
    NSToolbarSpaceItemIdentifier,
)
from FilteringArrayController import kLiteralSearch, kRegularExpressionSearch

kToolbarIdentifier = "TableModel Toolbar Identifier"
kSearchFieldItemIdentifier = "TableModel Search Field Identifier"


class ToolbarCreator(NSObject):
    filteringArrayController = objc.IBOutlet()
    searchField = objc.IBOutlet()
    window = objc.IBOutlet()

    def awakeFromNib(self):
        self.toolbarItemCache = {}

        # create toolbar containing search field
        toolbar = NSToolbar.alloc().initWithIdentifier_(kToolbarIdentifier)
        toolbar.setDelegate_(self)
        toolbar.setAllowsUserCustomization_(True)
        toolbar.setAutosavesConfiguration_(True)

        searchFieldItem = NSToolbarItem.alloc().initWithItemIdentifier_(
            kSearchFieldItemIdentifier
        )
        self.searchFieldItem = searchFieldItem
        searchFieldItem.setLabel_("Search")
        searchFieldItem.setPaletteLabel_("Search Field")
        searchFieldItem.setToolTip_("Search")
        searchFieldItem.setView_(self.searchField)
        searchFieldItem.setMinSize_(self.searchField.bounds().size)
        maxSize = self.searchField.bounds().size
        maxSize.width = maxSize.width + 150
        searchFieldItem.setMaxSize_(maxSize)

        self.toolbarItemCache[kSearchFieldItemIdentifier] = searchFieldItem

        self.window.setToolbar_(toolbar)

        cellMenu = NSMenu.alloc().initWithTitle_("Search Menu")
        # note, bottom up!
        for v in [kRegularExpressionSearch, kLiteralSearch]:
            item = NSMenuItem.alloc().initWithTitle_action_keyEquivalent_(
                v, "changeSearchType:", ""
            )
            item.setRepresentedObject_(v)
            item.setTarget_(self)
            cellMenu.insertItem_atIndex_(item, 0)
        self.searchField.cell().setSearchMenuTemplate_(cellMenu)
        # this better be the kLiteralSearch menu item
        self.changeSearchType_(item)

    @objc.IBAction
    def changeSearchType_(self, sender):
        obj = sender.representedObject()
        self.searchField.cell().setPlaceholderString_(obj)
        self.searchField.setStringValue_("")
        self.filteringArrayController.changeSearchType_(obj)

    def toolbarDefaultItemIdentifiers_(self, aToolbar):
        return [
            kSearchFieldItemIdentifier,
            NSToolbarFlexibleSpaceItemIdentifier,
            NSToolbarSeparatorItemIdentifier,
            NSToolbarCustomizeToolbarItemIdentifier,
        ]

    def toolbarAllowedItemIdentifiers_(self, aToolbar):
        return [
            kSearchFieldItemIdentifier,
            NSToolbarFlexibleSpaceItemIdentifier,
            NSToolbarSpaceItemIdentifier,
            NSToolbarSeparatorItemIdentifier,
            NSToolbarPrintItemIdentifier,
            NSToolbarCustomizeToolbarItemIdentifier,
        ]

    def toolbar_itemForItemIdentifier_willBeInsertedIntoToolbar_(
        self, toolbar, itemIdentifier, flag
    ):
        newItem = NSToolbarItem.alloc().initWithItemIdentifier_(itemIdentifier)
        item = self.toolbarItemCache[itemIdentifier]

        newItem.setLabel_(item.label())
        newItem.setPaletteLabel_(item.paletteLabel())
        if item.view():
            newItem.setView_(item.view())
        else:
            newItem.setImage_(item.image())

        newItem.setToolTip_(item.toolTip())
        newItem.setTarget_(item.target())
        newItem.setAction_(item.action())
        newItem.setMenuFormRepresentation_(item.menuFormRepresentation())

        if newItem.view():
            newItem.setMinSize_(item.minSize())
            newItem.setMaxSize_(item.maxSize())

        return newItem



import CoreFoundation as _CF
import objc as _objc


#
# 'Emulation' for CFArray constructors
#
def _setup():
    NSArray = _objc.lookUpClass("NSArray")
    NSMutableArray = _objc.lookUpClass("NSMutableArray")

    def CFArrayCreate(allocator, values, numvalues, callbacks):
        assert callbacks is None
        return NSArray.alloc().initWithArray_(values[:numvalues])

    def CFArrayCreateMutable(allocator, capacity, callbacks):
        assert callbacks is None
        return NSMutableArray.alloc().init()

    return CFArrayCreate, CFArrayCreateMutable


CFArrayCreate, CFArrayCreateMutable = _setup()

# CFDictionary emulation functions


def _setup():
    NSDictionary = _objc.lookUpClass("NSDictionary")
    NSMutableDictionary = _objc.lookUpClass("NSMutableDictionary")

    def CFDictionaryCreate(
        allocator, keys, values, numValues, keyCallbacks, valueCallbacks
    ):
        assert keyCallbacks is None
        assert valueCallbacks is None

        keys = list(keys)[:numValues]
        values = list(values)[:numValues]

        return NSDictionary.dictionaryWithDictionary_(dict(zip(keys, values)))

    def CFDictionaryCreateMutable(allocator, capacity, keyCallbacks, valueCallbacks):
        assert keyCallbacks is None
        assert valueCallbacks is None

        return NSMutableDictionary.dictionary()

    return CFDictionaryCreate, CFDictionaryCreateMutable


CFDictionaryCreate, CFDictionaryCreateMutable = _setup()


# CFSet emulation functions


def _setup():
    NSSet = _objc.lookUpClass("NSSet")
    NSMutableSet = _objc.lookUpClass("NSMutableSet")

    def CFSetCreate(allocator, values, numvalues, callbacks):
        assert callbacks is None
        return NSSet.alloc().initWithArray_(values[:numvalues])

    def CFSetCreateMutable(allocator, capacity, callbacks):
        assert callbacks is None
        return NSMutableSet.alloc().init()

    return CFSetCreate, CFSetCreateMutable


CFSetCreate, CFSetCreateMutable = _setup()

kCFTypeArrayCallBacks = None
kCFTypeDictionaryKeyCallBacks = None
kCFTypeDictionaryValueCallBacks = None
kCFTypeSetCallBacks = None


#
# Implementation of a number of macro's in the CFBundle API
#


def CFCopyLocalizedString(key, comment):
    return _CF.CFBundleCopyLocalizedString(
        _CF.CFBundleGetMainBundle(), (key), (key), None
    )


def CFCopyLocalizedStringFromTable(key, tbl, comment):
    return _CF.CFBundleCopyLocalizedString(
        _CF.CFBundleGetMainBundle(), (key), (key), (tbl)
    )


def CFCopyLocalizedStringFromTableInBundle(key, tbl, bundle, comment):
    return _CF.CFBundleCopyLocalizedString((bundle), (key), (key), (tbl))


def CFCopyLocalizedStringWithDefaultValue(key, tbl, bundle, value, comment):
    return _CF.CFBundleCopyLocalizedString((bundle), (key), (value), (tbl))


def CFSTR(strval):
    return _objc.lookUpClass("NSString").stringWithString_(strval)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSPopUpButton(TestCase):
    def testConstants(self):
        self.assertIsInstance(AppKit.NSPopUpButtonWillPopUpNotification, str)

    def testMethods(self):
        self.assertArgIsBOOL(AppKit.NSPopUpButton.initWithFrame_pullsDown_, 1)
        self.assertResultIsBOOL(AppKit.NSPopUpButton.pullsDown)
        self.assertArgIsBOOL(AppKit.NSPopUpButton.setPullsDown_, 0)
        self.assertResultIsBOOL(AppKit.NSPopUpButton.autoenablesItems)
        self.assertArgIsBOOL(AppKit.NSPopUpButton.setAutoenablesItems_, 0)
        self.assertResultIsBOOL(AppKit.NSPopUpButton.selectItemWithTag_)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSMatrix(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSRadioModeMatrix, 0)
        self.assertEqual(AppKit.NSHighlightModeMatrix, 1)
        self.assertEqual(AppKit.NSListModeMatrix, 2)
        self.assertEqual(AppKit.NSTrackModeMatrix, 3)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSMatrix.allowsEmptySelection)
        self.assertArgIsBOOL(AppKit.NSMatrix.setAllowsEmptySelection_, 0)
        self.assertArgIsBOOL(AppKit.NSMatrix.sendAction_to_forAllCells_, 2)
        self.assertArgIsSEL(
            AppKit.NSMatrix.sendAction_to_forAllCells_, 0, objc._C_NSBOOL + b"@:@"
        )
        self.assertArgIsSEL(
            AppKit.NSMatrix.sortUsingSelector_, 0, objc._C_NSInteger + b"@:@"
        )
        self.assertArgIsOut(AppKit.NSMatrix.getNumberOfRows_columns_, 0)
        self.assertArgIsOut(AppKit.NSMatrix.getNumberOfRows_columns_, 1)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_ofCell_, 0)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_ofCell_, 1)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_forPoint_, 0)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_forPoint_, 1)

        self.assertArgIsFunction(
            AppKit.NSMatrix.sortUsingFunction_context_,
            0,
            objc._C_NSInteger + b"@@@",
            False,
        )
        self.assertArgHasType(AppKit.NSMatrix.sortUsingFunction_context_, 1, objc._C_ID)

        self.assertResultIsBOOL(AppKit.NSMatrix.isSelectionByRect)
        self.assertArgIsBOOL(AppKit.NSMatrix.setSelectionByRect_, 0)
        self.assertArgIsBOOL(AppKit.NSMatrix.setSelectionFrom_to_anchor_highlight_, 3)
        self.assertResultIsBOOL(AppKit.NSMatrix.selectCellWithTag_)
        self.assertArgIsBOOL(AppKit.NSMatrix.setScrollable_, 0)
        self.assertArgIsBOOL(AppKit.NSMatrix.setDrawsCellBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSMatrix.drawsCellBackground)
        self.assertArgIsBOOL(AppKit.NSMatrix.setDrawsBackground_, 0)
        self.assertResultIsBOOL(AppKit.NSMatrix.drawsBackground)
        self.assertArgIsOut(AppKit.NSMatrix.getNumberOfRows_columns_, 0)
        self.assertArgIsOut(AppKit.NSMatrix.getNumberOfRows_columns_, 1)
        self.assertResultIsBOOL(AppKit.NSMatrix.getRow_column_ofCell_)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_ofCell_, 0)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_ofCell_, 1)
        self.assertResultIsBOOL(AppKit.NSMatrix.getRow_column_forPoint_)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_forPoint_, 0)
        self.assertArgIsOut(AppKit.NSMatrix.getRow_column_forPoint_, 1)
        self.assertArgIsBOOL(AppKit.NSMatrix.setAutosizesCells_, 0)
        self.assertResultIsBOOL(AppKit.NSMatrix.autosizesCells)
        self.assertArgIsBOOL(AppKit.NSMatrix.setValidateSize_, 0)
        self.assertArgIsBOOL(AppKit.NSMatrix.highlightCell_atRow_column_, 0)
        self.assertArgIsBOOL(AppKit.NSMatrix.setAutoscroll_, 0)
        self.assertResultIsBOOL(AppKit.NSMatrix.isAutoscroll)
        self.assertResultIsBOOL(AppKit.NSMatrix.performKeyEquivalent_)
        self.assertResultIsBOOL(AppKit.NSMatrix.sendAction)
        self.assertResultIsBOOL(AppKit.NSMatrix.textShouldBeginEditing_)
        self.assertResultIsBOOL(AppKit.NSMatrix.textShouldEndEditing_)
        self.assertResultIsBOOL(AppKit.NSMatrix.acceptsFirstMouse_)
        self.assertArgIsBOOL(AppKit.NSMatrix.setTabKeyTraversesCells_, 0)
        self.assertResultIsBOOL(AppKit.NSMatrix.tabKeyTraversesCells)

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertArgIsBOOL(AppKit.NSMatrix.setAutorecalculatesCellSize_, 0)
        self.assertResultIsBOOL(AppKit.NSMatrix.autorecalculatesCellSize)

    @min_sdk_level("10.6")
    def testProtocols(self):
        objc.protocolNamed("NSMatrixDelegate")



import Foundation
from PyObjCTools.TestSupport import TestCase


class TestXMLDTD(TestCase):
    def testOutputArgs(self):
        self.assertArgIsOut(Foundation.NSXMLDTD.initWithContentsOfURL_options_error_, 2)
        self.assertArgIsOut(Foundation.NSXMLDTD.initWithData_options_error_, 2)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFileVersion(TestCase):
    @min_os_level("10.7")
    def testContants10_7(self):
        self.assertEqual(Foundation.NSFileVersionAddingByMoving, 1 << 0)
        self.assertEqual(Foundation.NSFileVersionReplacingByMoving, 1 << 0)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsOut(
            Foundation.NSFileVersion.addVersionOfItemAtURL_withContentsOfURL_options_error_,
            3,
        )
        self.assertArgIsOut(Foundation.NSFileVersion.replaceItemAtURL_options_error_, 2)
        self.assertArgIsOut(Foundation.NSFileVersion.removeAndReturnError_, 0)
        self.assertResultIsBOOL(Foundation.NSFileVersion.removeAndReturnError_)

        self.assertArgIsOut(
            Foundation.NSFileVersion.removeOtherVersionsOfItemAtURL_error_, 1
        )
        self.assertResultIsBOOL(
            Foundation.NSFileVersion.removeOtherVersionsOfItemAtURL_error_
        )

        self.assertResultIsBOOL(Foundation.NSFileVersion.isConflict)

        self.assertArgIsBOOL(Foundation.NSFileVersion.setResolved_, 0)
        self.assertResultIsBOOL(Foundation.NSFileVersion.isResolved)
        self.assertArgIsBOOL(Foundation.NSFileVersion.setDiscardable_, 0)
        self.assertResultIsBOOL(Foundation.NSFileVersion.isDiscardable)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSFileVersion.getNonlocalVersionsOfItemAtURL_completionHandler_,
            1,
            b"v@@",
        )
        self.assertResultIsBOOL(Foundation.NSFileVersion.hasLocalContents)
        self.assertResultIsBOOL(Foundation.NSFileVersion.hasThumbnail)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSMetaData(TestCase):
    def testConstants(self):
        self.assertIsInstance(
            Foundation.NSMetadataQueryDidStartGatheringNotification, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataQueryGatheringProgressNotification, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataQueryDidFinishGatheringNotification, str
        )
        self.assertIsInstance(Foundation.NSMetadataQueryDidUpdateNotification, str)
        self.assertIsInstance(
            Foundation.NSMetadataQueryResultContentRelevanceAttribute, str
        )
        self.assertIsInstance(Foundation.NSMetadataQueryUserHomeScope, str)
        self.assertIsInstance(Foundation.NSMetadataQueryLocalComputerScope, str)
        self.assertIsInstance(Foundation.NSMetadataQueryNetworkScope, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(Foundation.NSMetadataQueryLocalDocumentsScope, str)
        self.assertIsInstance(Foundation.NSMetadataQueryUbiquitousDocumentsScope, str)
        self.assertIsInstance(Foundation.NSMetadataQueryUbiquitousDataScope, str)

        self.assertIsInstance(Foundation.NSMetadataItemFSNameKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemDisplayNameKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemURLKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemPathKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFSSizeKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFSCreationDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemFSContentChangeDateKey, str)
        self.assertIsInstance(Foundation.NSMetadataItemIsUbiquitousKey, str)
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemHasUnresolvedConflictsKey, str
        )
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsDownloadedKey, str)
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsDownloadingKey, str)
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsUploadedKey, str)
        self.assertIsInstance(Foundation.NSMetadataUbiquitousItemIsUploadingKey, str)
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemPercentDownloadedKey, str
        )
        self.assertIsInstance(
            Foundation.NSMetadataUbiquitousItemPercentUploadedKey, str
        )

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(Foundation.NSMetadataQueryUpdateAddedItemsKey, str)
        self.assertIsInstance(Foundation.NSMetadataQueryUpdateChangedItemsKey, str)
        self.assertIsInstance(Foundation.NSMetadataQueryUpdateRemovedItemsKey, str)
        self.assertIsInstance(Foundation.NSMetadataQueryIndexedLocalComputerScope, str)
        self.assertIsInstance(Foundation.NSMetadataQueryIndexedNetworkScope, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertIsInstance(
            Foundation.NSMetadataQueryAccessibleUbiquitousExternalDocumentsScope, str
        )

    def testMethods(self):
        self.assertResultIsBOOL(Foundation.NSMetadataQuery.startQuery)
        self.assertResultIsBOOL(Foundation.NSMetadataQuery.isStarted)
        self.assertResultIsBOOL(Foundation.NSMetadataQuery.isGathering)
        self.assertResultIsBOOL(Foundation.NSMetadataQuery.isStopped)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBlock(
            Foundation.NSMetadataQuery.enumerateResultsUsingBlock_,
            0,
            b"v@" + objc._C_NSUInteger + b"o^Z",
        )
        self.assertArgIsBlock(
            Foundation.NSMetadataQuery.enumerateResultsWithOptions_usingBlock_,
            1,
            b"v@" + objc._C_NSUInteger + b"o^Z",
        )

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSMetadataQueryDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPersistentDocument(TestCase):
    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(
            AppKit.NSPersistentDocument.configurePersistentStoreCoordinatorForURL_ofType_modelConfiguration_storeOptions_error_  # noqa: B950
        )
        self.assertArgIsOut(
            AppKit.NSPersistentDocument.configurePersistentStoreCoordinatorForURL_ofType_modelConfiguration_storeOptions_error_,  # noqa: B950
            4,
        )

    def testMethods(self):
        self.assertResultIsBOOL(
            AppKit.NSPersistentDocument.writeToURL_ofType_forSaveOperation_originalContentsURL_error_  # noqa: B950
        )
        self.assertArgIsOut(
            AppKit.NSPersistentDocument.writeToURL_ofType_forSaveOperation_originalContentsURL_error_,  # noqa: B950
            4,
        )

        self.assertResultIsBOOL(AppKit.NSPersistentDocument.readFromURL_ofType_error_)
        self.assertArgIsOut(AppKit.NSPersistentDocument.readFromURL_ofType_error_, 2)

        self.assertResultIsBOOL(
            AppKit.NSPersistentDocument.revertToContentsOfURL_ofType_error_
        )
        self.assertArgIsOut(
            AppKit.NSPersistentDocument.revertToContentsOfURL_ofType_error_, 2
        )

        self.assertResultIsBOOL(
            AppKit.NSPersistentDocument.configurePersistentStoreCoordinatorForURL_ofType_error_
        )
        self.assertArgIsOut(
            AppKit.NSPersistentDocument.configurePersistentStoreCoordinatorForURL_ofType_error_,
            2,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSGestureRecognizerHelper(AppKit.NSObject):
    def gestureRecognizerShouldBegin_(self, g):
        return 1

    def gestureRecognizer_shouldRecognizeSimultaneouslyWithGestureRecognizer_(
        self, g, a
    ):
        return 1

    def gestureRecognizer_shouldRequireFailureOfGestureRecognizer_(self, g, a):
        return 1

    def gestureRecognizer_shouldBeRequiredToFailByGestureRecognizer_(self, g, a):
        return 1

    def gestureRecognizer_shouldAttemptToRecognizeWithEvent_(self, g, e):
        return 1

    def gestureRecognizer_shouldReceiveTouch_(self, r, t):
        return 1


class TestNSGestureRecognizer(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSGestureRecognizerStatePossible, 0)
        self.assertEqual(AppKit.NSGestureRecognizerStateBegan, 1)
        self.assertEqual(AppKit.NSGestureRecognizerStateChanged, 2)
        self.assertEqual(AppKit.NSGestureRecognizerStateEnded, 3)
        self.assertEqual(AppKit.NSGestureRecognizerStateCancelled, 4)
        self.assertEqual(AppKit.NSGestureRecognizerStateFailed, 5)
        self.assertEqual(
            AppKit.NSGestureRecognizerStateRecognized,
            AppKit.NSGestureRecognizerStateEnded,
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsSEL(
            AppKit.NSGestureRecognizer.initWithTarget_action_, 1, b"v@:@"
        )
        self.assertArgIsSEL(AppKit.NSGestureRecognizer.setAction_, 0, b"v@:@")
        self.assertResultIsBOOL(AppKit.NSGestureRecognizer.isEnabled)
        self.assertArgIsBOOL(AppKit.NSGestureRecognizer.setEnabled_, 0)

        self.assertResultIsBOOL(
            AppKit.NSGestureRecognizer.delaysPrimaryMouseButtonEvents
        )
        self.assertResultIsBOOL(
            AppKit.NSGestureRecognizer.delaysSecondaryMouseButtonEvents
        )
        self.assertResultIsBOOL(AppKit.NSGestureRecognizer.delaysOtherMouseButtonEvents)
        self.assertResultIsBOOL(AppKit.NSGestureRecognizer.delaysKeyEvents)
        self.assertResultIsBOOL(AppKit.NSGestureRecognizer.delaysMagnificationEvents)
        self.assertResultIsBOOL(AppKit.NSGestureRecognizer.delaysRotationEvents)

        self.assertArgIsBOOL(
            AppKit.NSGestureRecognizer.setDelaysPrimaryMouseButtonEvents_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSGestureRecognizer.setDelaysSecondaryMouseButtonEvents_, 0
        )
        self.assertArgIsBOOL(
            AppKit.NSGestureRecognizer.setDelaysOtherMouseButtonEvents_, 0
        )
        self.assertArgIsBOOL(AppKit.NSGestureRecognizer.setDelaysKeyEvents_, 0)
        self.assertArgIsBOOL(
            AppKit.NSGestureRecognizer.setDelaysMagnificationEvents_, 0
        )
        self.assertArgIsBOOL(AppKit.NSGestureRecognizer.setDelaysRotationEvents_, 0)

        self.assertResultIsBOOL(AppKit.NSGestureRecognizer.canPreventGestureRecognizer_)
        self.assertResultIsBOOL(
            AppKit.NSGestureRecognizer.canBePreventedByGestureRecognizer_
        )
        self.assertResultIsBOOL(
            AppKit.NSGestureRecognizer.shouldRequireFailureOfGestureRecognizer_
        )
        self.assertResultIsBOOL(
            AppKit.NSGestureRecognizer.shouldBeRequiredToFailByGestureRecognizer_
        )

    @min_sdk_level("10.10")
    def testProtocols(self):
        objc.protocolNamed("NSGestureRecognizerDelegate")
        self.assertResultIsBOOL(
            TestNSGestureRecognizerHelper.gestureRecognizerShouldBegin_
        )
        self.assertResultIsBOOL(
            TestNSGestureRecognizerHelper.gestureRecognizer_shouldRecognizeSimultaneouslyWithGestureRecognizer_  # noqa: B950
        )
        self.assertResultIsBOOL(
            TestNSGestureRecognizerHelper.gestureRecognizer_shouldRequireFailureOfGestureRecognizer_  # noqa: B950
        )
        self.assertResultIsBOOL(
            TestNSGestureRecognizerHelper.gestureRecognizer_shouldBeRequiredToFailByGestureRecognizer_  # noqa: B950
        )

    @min_sdk_level("10.11")
    def testProtocols10_11(self):
        self.assertResultIsBOOL(
            TestNSGestureRecognizerHelper.gestureRecognizer_shouldAttemptToRecognizeWithEvent_
        )

    @min_sdk_level("10.12")
    def testProtocols10_12(self):
        self.assertResultIsBOOL(
            TestNSGestureRecognizerHelper.gestureRecognizer_shouldReceiveTouch_
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSTokenFieldHelper(AppKit.NSObject):
    def tokenField_completionsForSubstring_indexOfToken_indexOfSelectedItem_(
        self, a, b, c, d
    ):
        return 1

    def tokenField_shouldAddObjects_atIndex_(self, a, b, c):
        return 1

    def tokenField_writeRepresentedObjects_toPasteboard_(self, a, b, c):
        return 1

    def tokenField_hasMenuForRepresentedObject_(self, a, b):
        return 1

    def tokenField_styleForRepresentedObject_(self, a, b):
        return 1


class TestNSTokenField(TestCase):
    @min_sdk_level("10.7")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTokenFieldDelegate")

    def testProtocols(self):
        self.assertArgHasType(
            TestNSTokenFieldHelper.tokenField_completionsForSubstring_indexOfToken_indexOfSelectedItem_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTokenFieldHelper.tokenField_completionsForSubstring_indexOfToken_indexOfSelectedItem_,  # noqa: B950
            3,
            b"o^" + objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSTokenFieldHelper.tokenField_shouldAddObjects_atIndex_,
            2,
            objc._C_NSUInteger,
        )
        self.assertResultIsBOOL(
            TestNSTokenFieldHelper.tokenField_writeRepresentedObjects_toPasteboard_
        )
        self.assertResultIsBOOL(
            TestNSTokenFieldHelper.tokenField_hasMenuForRepresentedObject_
        )
        self.assertResultHasType(
            TestNSTokenFieldHelper.tokenField_styleForRepresentedObject_,
            objc._C_NSUInteger,
        )



import CoreFoundation
from PyObjCTools.TestSupport import TestCase, os_level_key, os_release, min_os_level


class TestCFBundle(TestCase):
    def testTypes(self):
        self.assertIsCFType(CoreFoundation.CFBundleRef)

    def testMainBundle(self):
        bundle = CoreFoundation.CFBundleGetMainBundle()
        self.assertIsInstance(bundle, CoreFoundation.CFBundleRef)

    def testBundleLoader(self):
        bundle = CoreFoundation.CFBundleGetBundleWithIdentifier(
            "com.apple.CoreFoundation"
        )
        self.assertIsInstance(bundle, CoreFoundation.CFBundleRef)
        array = list(CoreFoundation.CFBundleGetAllBundles())
        self.assertNotEqual(len(array), 0)
        for b in array:
            self.assertIsInstance(b, CoreFoundation.CFBundleRef)
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Frameworks/Foundation.framework",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )
        bundle = CoreFoundation.CFBundleCreate(None, url)
        self.assertIsInstance(bundle, CoreFoundation.CFBundleRef)
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Frameworks",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )
        array = CoreFoundation.CFBundleCreateBundlesFromDirectory(
            None, url, "frameworkX"
        )
        self.assertEqual(len(array), 0)

        array = CoreFoundation.CFBundleCreateBundlesFromDirectory(
            None, url, "framework"
        )
        self.assertNotEqual(len(array), 0)

        array = CoreFoundation.CFBundleCreateBundlesFromDirectory(None, url, None)
        self.assertNotEqual(len(array), 0)

    def testTypeID(self):
        v = CoreFoundation.CFBundleGetTypeID()
        self.assertIsInstance(v, int)

    def testInspection(self):
        bundle = CoreFoundation.CFBundleGetBundleWithIdentifier(
            "com.apple.CoreFoundation"
        )
        self.assertIsInstance(bundle, CoreFoundation.CFBundleRef)
        url = CoreFoundation.CFBundleCopyBundleURL(bundle)
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        v = CoreFoundation.CFBundleGetValueForInfoDictionaryKey(
            bundle, "CFBundleIdentifier"
        )
        self.assertEqual(v, "com.apple.CoreFoundation")
        v = CoreFoundation.CFBundleGetInfoDictionary(bundle)
        self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        self.assertTrue(v["CFBundleIdentifier"], "com.apple.CoreFoundation")

        v = CoreFoundation.CFBundleGetLocalInfoDictionary(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        package_type, creator = CoreFoundation.CFBundleGetPackageInfo(
            bundle, None, None
        )
        self.assertIsInstance(package_type, int)
        self.assertIsInstance(creator, int)
        identifier = CoreFoundation.CFBundleGetIdentifier(bundle)
        self.assertEqual(identifier, "com.apple.CoreFoundation")
        v = CoreFoundation.CFBundleGetVersionNumber(bundle)
        self.assertIsInstance(v, int)
        v = CoreFoundation.CFBundleGetDevelopmentRegion(bundle)
        self.assertIsInstance(v, str)
        v = CoreFoundation.CFBundleCopySupportFilesDirectoryURL(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFURLRef)
        v = CoreFoundation.CFBundleCopyResourcesDirectoryURL(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFURLRef)
        v = CoreFoundation.CFBundleCopyPrivateFrameworksURL(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFURLRef)
        v = CoreFoundation.CFBundleCopySharedFrameworksURL(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFURLRef)
        v = CoreFoundation.CFBundleCopySharedSupportURL(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFURLRef)
        v = CoreFoundation.CFBundleCopyBuiltInPlugInsURL(bundle)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFURLRef)

    def testDirectAccess(self):
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Frameworks/Foundation.framework",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )

        v = CoreFoundation.CFBundleCopyInfoDictionaryInDirectory(url)
        self.assertIsInstance(v, CoreFoundation.CFDictionaryRef)
        self.assertEqual(v["CFBundleIdentifier"], "com.apple.Foundation")
        ok, bundle_type, creator = CoreFoundation.CFBundleGetPackageInfoInDirectory(
            url, None, None
        )
        self.assertIs(ok, True)
        self.assertIsInstance(bundle_type, int)
        self.assertIsInstance(creator, int)

    def testResources(self):
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Frameworks/Foundation.framework",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )
        bundle = CoreFoundation.CFBundleCreate(None, url)
        self.assertIsInstance(bundle, CoreFoundation.CFBundleRef)
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Frameworks/Tcl.framework",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )
        bundle2 = CoreFoundation.CFBundleCreate(None, url)
        self.assertIsInstance(bundle2, CoreFoundation.CFBundleRef)
        url = CoreFoundation.CFBundleCopyResourceURL(
            bundle, "Formatter", "strings", None
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        url = CoreFoundation.CFBundleCopyResourceURL(
            bundle, "Formatter", "strings", "helloworld.lproj"
        )
        self.assertIs(url, None)
        array = CoreFoundation.CFBundleCopyResourceURLsOfType(bundle, "strings", None)
        self.assertIsNot(array, None)
        self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        val = CoreFoundation.CFBundleCopyLocalizedString(
            bundle, "Err640.f", "value", "FoundationErrors"
        )
        self.assertIsInstance(val, str)
        self.assertNotEqual(val, "value")
        CoreFoundation.CFCopyLocalizedString("python", "error")
        CoreFoundation.CFCopyLocalizedStringFromTable("pyobjc", "python", "error")
        CoreFoundation.CFCopyLocalizedStringFromTableInBundle(
            "pyobjc", "python", bundle, "comment"
        )
        CoreFoundation.CFCopyLocalizedStringWithDefaultValue(
            "pyobjc", "python", bundle, "default", "comment"
        )

        array = CoreFoundation.CFBundleCopyBundleLocalizations(bundle)
        self.assertIsNot(array, None)
        self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        arr2 = CoreFoundation.CFBundleCopyPreferredLocalizationsFromArray(array)
        self.assertIsNot(arr2, None)
        self.assertIsInstance(arr2, CoreFoundation.CFArrayRef)
        arr2 = CoreFoundation.CFBundleCopyLocalizationsForPreferences(array, None)
        self.assertIsNot(arr2, None)
        self.assertIsInstance(arr2, CoreFoundation.CFArrayRef)
        url = CoreFoundation.CFBundleCopyResourceURLForLocalization(
            bundle, "Formatter", "strings", None, "Dutch"
        )
        if url is None:
            url = CoreFoundation.CFBundleCopyResourceURLForLocalization(
                bundle, "Formatter", "strings", None, "nl"
            )
        if url is None:
            url = CoreFoundation.CFBundleCopyResourceURLForLocalization(
                bundle, "Formatter", "strings", None, "en"
            )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)

        array = CoreFoundation.CFBundleCopyResourceURLsOfTypeForLocalization(
            bundle, "strings", None, "Dutch"
        )
        self.assertIsNot(array, None)
        self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        url = CoreFoundation.CFBundleCopyExecutableURL(bundle)
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        array = CoreFoundation.CFBundleCopyExecutableArchitectures(bundle)
        if os_level_key(os_release()) >= os_level_key("11.0"):
            self.assertIs(array, None)
        else:
            self.assertIsNot(array, None)
            self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        self.assertArgIsOut(CoreFoundation.CFBundlePreflightExecutable, 1)
        ok, error = CoreFoundation.CFBundlePreflightExecutable(bundle, None)
        self.assertTrue((ok is True) or (ok is False))
        if ok:
            self.assertIs(error, None)
        else:
            self.assertIsInstance(error, CoreFoundation.CFErrorRef)
        self.assertArgIsOut(CoreFoundation.CFBundleLoadExecutableAndReturnError, 1)
        ok, error = CoreFoundation.CFBundleLoadExecutableAndReturnError(bundle2, None)
        self.assertTrue((ok is True) or (ok is False))
        if ok:
            self.assertIs(error, None)
        else:
            self.assertIsInstance(error, CoreFoundation.CFErrorRef)
        ok = CoreFoundation.CFBundleLoadExecutable(bundle2)
        self.assertTrue(ok)

        ok = CoreFoundation.CFBundleIsExecutableLoaded(bundle2)
        self.assertTrue(ok)

        CoreFoundation.CFBundleUnloadExecutable(bundle2)
        ok = CoreFoundation.CFBundleIsExecutableLoaded(bundle2)
        # self.assertFalse(ok)
        ok = CoreFoundation.CFBundleLoadExecutable(bundle2)
        self.assertTrue(ok)

        try:
            CoreFoundation.CFBundleGetFunctionPointerForName
        except AttributeError:
            pass
        else:
            self.fail("CFBundleGetFunctionPointerForName")

        try:
            CoreFoundation.CFBundleGetFunctionPointersForNames
        except AttributeError:
            pass
        else:
            self.fail("CFBundleGetFunctionPointersForNames")

        try:
            CoreFoundation.CFBundleGetDataPointerForName
        except AttributeError:
            pass
        else:
            self.fail("CFBundleGetDataPointerForName")

        try:
            CoreFoundation.CFBundleGetDataPointersForNames
        except AttributeError:
            pass
        else:
            self.fail("CFBundleGetDataPointersForNames")

        url = CoreFoundation.CFBundleCopyAuxiliaryExecutableURL(bundle, "Foundation")
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        map_id = CoreFoundation.CFBundleOpenBundleResourceMap(bundle)
        self.assertIsInstance(map_id, int)
        CoreFoundation.CFBundleCloseBundleResourceMap(bundle, map_id)

        err, id1, id2 = CoreFoundation.CFBundleOpenBundleResourceFiles(
            bundle, None, None
        )
        self.assertIsInstance(err, int)
        self.assertIsInstance(id1, int)
        self.assertIsInstance(id2, int)
        if id1 != -1:
            CoreFoundation.CFBundleCloseBundleResourceMap(bundle, id1)
        if id2 != -1:
            CoreFoundation.CFBundleCloseBundleResourceMap(bundle, id2)

    def testResourcesDirect(self):
        bundle = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Frameworks/Foundation.framework",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )
        url = CoreFoundation.CFBundleCopyResourceURLInDirectory(
            bundle, "Formatter", "strings", None
        )
        self.assertIsInstance(url, CoreFoundation.CFURLRef)
        array = CoreFoundation.CFBundleCopyResourceURLsOfTypeInDirectory(
            bundle, "strings", None
        )
        self.assertIsNot(array, None)
        self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        infoDict = CoreFoundation.CFBundleCopyInfoDictionaryForURL(bundle)
        self.assertIsInstance(infoDict, CoreFoundation.CFDictionaryRef)
        array = CoreFoundation.CFBundleCopyLocalizationsForURL(bundle)
        self.assertIsInstance(array, CoreFoundation.CFArrayRef)
        array = CoreFoundation.CFBundleCopyExecutableArchitecturesForURL(bundle)
        if os_level_key(os_release()) >= os_level_key("11.0"):
            self.assertIs(array, None)
        else:
            self.assertIsInstance(array, CoreFoundation.CFArrayRef)
            for a in array:
                self.assertIsInstance(a, int)

    def testPlugin(self):
        url = CoreFoundation.CFURLCreateWithFileSystemPath(
            None,
            "/System/Library/Components/AppleScript.component",
            CoreFoundation.kCFURLPOSIXPathStyle,
            True,
        )
        bundle = CoreFoundation.CFBundleCreate(None, url)
        self.assertIsInstance(bundle, CoreFoundation.CFBundleRef)
        ref = CoreFoundation.CFBundleGetPlugIn(bundle)
        self.assertIs(ref, None)

    def testConstants(self):
        self.assertIsInstance(CoreFoundation.kCFBundleInfoDictionaryVersionKey, str)
        self.assertIsInstance(CoreFoundation.kCFBundleExecutableKey, str)
        self.assertIsInstance(CoreFoundation.kCFBundleIdentifierKey, str)
        self.assertIsInstance(CoreFoundation.kCFBundleVersionKey, str)
        self.assertIsInstance(CoreFoundation.kCFBundleDevelopmentRegionKey, str)
        self.assertIsInstance(CoreFoundation.kCFBundleNameKey, str)
        self.assertIsInstance(CoreFoundation.kCFBundleLocalizationsKey, str)
        self.assertEqual(CoreFoundation.kCFBundleExecutableArchitectureI386, 0x00000007)
        self.assertEqual(CoreFoundation.kCFBundleExecutableArchitecturePPC, 0x00000012)
        self.assertEqual(
            CoreFoundation.kCFBundleExecutableArchitectureX86_64, 0x01000007
        )
        self.assertEqual(
            CoreFoundation.kCFBundleExecutableArchitecturePPC64, 0x01000012
        )
        self.assertEqual(
            CoreFoundation.kCFBundleExecutableArchitectureARM64, 0x0100000C
        )

    @min_os_level("11.0")
    def test_functions11_0(self):
        self.assertResultIsBOOL(CoreFoundation.CFBundleIsExecutableLoadable)
        self.assertResultIsBOOL(CoreFoundation.CFBundleIsExecutableLoadableForURL)
        self.assertResultIsBOOL(CoreFoundation.CFBundleIsArchitectureLoadable)



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSGroupTouchBarItem(TestCase):
    @min_os_level("10.13")
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSGroupTouchBarItem.prefersEqualWidths)
        self.assertArgIsBOOL(AppKit.NSGroupTouchBarItem.setPrefersEqualWidths_, 0)



"""PythonBrowserModel.py -- module implementing the data model for PythonBrowser."""

import sys
from operator import getitem, setitem

from AppKit import NSBeep
from Foundation import NSObject


class PythonBrowserModel(NSObject):

    """This is a delegate as well as a data source for NSOutlineViews."""

    def initWithObject_(self, obj):
        self = self.init()
        self.setObject_(obj)
        return self

    def setObject_(self, obj):
        self.root = PythonItem("<root>", obj, None, None)

    # NSOutlineViewDataSource  methods

    def outlineView_numberOfChildrenOfItem_(self, view, item):
        if item is None:
            item = self.root
        return len(item)

    def outlineView_child_ofItem_(self, view, child, item):
        if item is None:
            item = self.root
        return item.getChild_(child)

    def outlineView_isItemExpandable_(self, view, item):
        if item is None:
            item = self.root
        return item.isExpandable()

    def outlineView_objectValueForTableColumn_byItem_(self, view, col, item):
        if item is None:
            item = self.root
        return getattr(item, col.identifier())

    def outlineView_setObjectValue_forTableColumn_byItem_(self, view, value, col, item):
        assert col.identifier() == "value"
        if item.value == value:
            return
        try:
            obj = eval(value, {})
        except:  # noqa: E722, B001
            NSBeep()
            print("Error:", sys.exc_info())
            print("     :", repr(value))
        else:
            item.setValue_(obj)

    # delegate method
    def outlineView_shouldEditTableColumn_item_(self, view, col, item):
        return item.isEditable()


# objects of these types are not eligible for expansion in the outline view
try:
    SIMPLE_TYPES = (str, unicode, int, long, float, complex)
except NameError:
    SIMPLE_TYPES = (str, int, float, complex)


def getInstanceVarNames(obj):
    """Return a list the names of all (potential) instance variables."""
    # Recipe from Guido
    slots = {}
    if hasattr(obj, "__dict__"):
        slots.update(obj.__dict__)
    if hasattr(obj, "__class__"):
        slots["__class__"] = 1
    cls = getattr(obj, "__class__", type(obj))
    if hasattr(cls, "__mro__"):
        for base in cls.__mro__:
            for name, value in base.__dict__.items():
                if (
                    hasattr(value, "__get__")
                    and not callable(value)
                    and hasattr(obj, name)
                ):
                    slots[name] = 1
    if "__dict__" in slots:
        del slots["__dict__"]
    slots = sorted(slots.keys())
    return slots


class NiceError:

    """Wrapper for an exception so we can display it nicely in the browser."""

    def __init__(self, exc_info):
        self.exc_info = exc_info

    def __repr__(self):
        from traceback import format_exception_only

        lines = format_exception_only(*self.exc_info[:2])
        assert len(lines) == 1
        error = lines[0].strip()
        return "*** error *** %s" % error


class PythonItem(NSObject):

    """Wrapper class for items to be displayed in the outline view."""

    # We keep references to all child items (once created). This is
    # necessary because NSOutlineView holds on to PythonItem instances
    # without retaining them. If we don't make sure they don't get
    # garbage collected, the app will crash. For the same reason this
    # class _must_ derive from NSObject, since otherwise autoreleased
    # proxies will be fed to NSOutlineView, which will go away too soon.

    def __new__(cls, *args, **kwargs):
        # "Pythonic" constructor
        return cls.alloc().init()

    def __init__(self, name, obj, parent, setvalue):
        self.realName = name
        self.name = str(name)
        self.parent = parent
        self._setValue = setvalue
        self.type = type(obj).__name__
        try:
            self.value = repr(obj)[:256]
            assert isinstance(self.value, str)
        except:  # noqa: E722, B001
            self.value = repr(NiceError(sys.exc_info()))
        self.object = obj
        self.childrenEditable = 0
        if isinstance(obj, dict):
            self.children = list(obj.keys())
            self.children.sort()
            self._getChild = getitem
            self._setChild = setitem
            self.childrenEditable = 1
        elif obj is None or isinstance(obj, SIMPLE_TYPES):
            self._getChild = None
            self._setChild = None
        elif isinstance(obj, (list, tuple)):
            self.children = range(len(obj))
            self._getChild = getitem
            self._setChild = setitem
            if isinstance(obj, list):
                self.childrenEditable = 1
        else:
            self.children = getInstanceVarNames(obj)
            self._getChild = getattr
            self._setChild = setattr
            self.childrenEditable = 1
        self._childRefs = {}

    def setValue_(self, value):
        self._setValue(self.parent, self.realName, value)
        self.__init__(self.realName, value, self.parent, self._setValue)

    def isEditable(self):
        return self._setValue is not None

    def isExpandable(self):
        return self._getChild is not None

    def getChild_(self, child):
        if child in self._childRefs:
            return self._childRefs[child]

        name = self.children[child]
        try:
            obj = self._getChild(self.object, name)
        except:  # noqa: E722, B001
            obj = NiceError(sys.exc_info())

        if self.childrenEditable:
            childObj = PythonItem(name, obj, self.object, self._setChild)
        else:
            childObj = PythonItem(name, obj, None, None)
        self._childRefs[child] = childObj
        return childObj

    def __len__(self):
        return len(self.children)



import Cocoa
import objc

NOT_DONE = 0
DONE = 1
DEFERRED = 2


class ToDoCell(Cocoa.NSButtonCell):

    __slots__ = ("_triState", "_doneImage", "_deferredImage", "_timeDue")

    def init(self):
        self._triState = NOT_DONE
        self._timeDue = None
        self._doneImage = None
        self._deferredImage = None

        Cocoa.NSButtonCell.initTextCell_(self, "")

        self.setType_(Cocoa.NSToggleButton)
        self.setImagePosition_(Cocoa.NSImageLeft)
        self.setBezelStyle_(Cocoa.NSShadowlessSquareBezelStyle)
        self.setFont_(Cocoa.NSFont.userFontOfSize_(10))
        self.setAlignment_(Cocoa.NSRightTextAlignment)

        self._doneImage = Cocoa.NSImage.imageNamed_("DoneMark")
        self._deferredImage = Cocoa.NSImage.imageNamed_("DeferredMark")
        return self

    @objc.typedAccessor(objc._C_INT)
    def setTriState_(self, newState):
        if newState > DEFERRED:
            self._triState = NOT_DONE
        else:
            self._triState = newState

        self.updateImage()

    @objc.typedAccessor(objc._C_INT)
    def triState(self):
        return self._triState

    def setState_(self, val):
        pass

    def state(self):
        if self._triState == DEFERRED:
            return DONE
        else:
            return self._triState

    def updateImage(self):

        if self._triState == NOT_DONE:
            self.setImage_(None)
        elif self._triState == DONE:
            self.setImage_(self._doneImage)
        elif self._triState == DEFERRED:
            self.setImage_(self._deferredImage)

        self.controlView().updateCell_(self)

    def startTrackingAt_inView_(self, startPoint, controlView):
        return 1

    def stopTracking_at_inView_mouseIsUp_(
        self, lastPoint, stopPoint, controlView, flag
    ):
        if flag:
            self.setTriState_(self.triState() + 1)

    def setTimeDue_(self, newTime):
        if newTime:
            self._timeDue = newTime
            self.setTitle_(
                self._timeDue.descriptionWithCalendarFormat_timeZone_locale_(
                    "%I:%M %p", Cocoa.NSTimeZone.localTimeZone(), None
                )
            )
        else:
            self._timeDue = None
            self.setTitle_("-->")

    def timeDue(self):
        return self._timeDue



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPickerTouchBarItem(TestCase):
    def test_contants(self):
        self.assertEqual(AppKit.NSPickerTouchBarItemSelectionModeSelectOne, 0)
        self.assertEqual(AppKit.NSPickerTouchBarItemSelectionModeSelectAny, 1)
        self.assertEqual(AppKit.NSPickerTouchBarItemSelectionModeMomentary, 2)

        self.assertEqual(AppKit.NSPickerTouchBarItemControlRepresentationAutomatic, 0)
        self.assertEqual(AppKit.NSPickerTouchBarItemControlRepresentationExpanded, 1)
        self.assertEqual(AppKit.NSPickerTouchBarItemControlRepresentationCollapsed, 2)

    @min_os_level("10.15")
    def test_nethods10_15(self):
        self.assertArgIsSEL(
            AppKit.NSPickerTouchBarItem.pickerTouchBarItemWithIdentifier_labels_selectionMode_target_action_,  # noqa: B950
            4,
            b"v@:@",
        )
        self.assertArgIsSEL(
            AppKit.NSPickerTouchBarItem.pickerTouchBarItemWithIdentifier_images_selectionMode_target_action_,  # noqa: B950
            4,
            b"v@:@",
        )

        self.assertArgIsSEL(AppKit.NSPickerTouchBarItem.setAction_, 0, b"v@:@")

        self.assertResultIsBOOL(AppKit.NSPickerTouchBarItem.isEnabled)
        self.assertArgIsBOOL(AppKit.NSPickerTouchBarItem.setEnabled_, 0)
        self.assertResultIsBOOL(AppKit.NSPickerTouchBarItem.isEnabledAtIndex_)
        self.assertArgIsBOOL(AppKit.NSPickerTouchBarItem.setEnabled_atIndex_, 0)
